C======================================================================
C  Routines to be called by the subroutine, sgcorr, for the soft-gluon 
C  correction to PS-applied events.
C  Those relevant to AA + 1 jet production are collected.
C  June 2, 2015; S.O.
C======================================================================
      subroutine sgcchk_aa1j( ppart, idpart, iret )
C  Check whether the correction should be applied or not.
C  iret = 0: Yes
C         1: No, because the event is including a soft radiation; e.g., 
C            the event corresponds to the non-log component.
      implicit real*8(a-h,o-z)
      real*8 ppart(4,10)
      integer idpart(10)
      real*8 bdir(3),ptemp(4),p0(4,10)
      parameter ( nextrn= 5, irad= 5 )

      iret= 1

      if( idpart(1).eq.21 .or. idpart(2).eq.21 ) then
         if( idpart(1).eq.21 ) then
            imoth= 1
            itarg= 2
         elseif( idpart(2).eq.21 ) then
            imoth= 2
            itarg= 1
         endif
         do i=1,4
            ptemp(i)= ppart(i,imoth) -ppart(i,irad)
         enddo
         qsq= ptemp(1)**2 +ptemp(2)**2 +ptemp(3)**2 -ptemp(4)**2
         call bstcm( ppart(1,itarg), ptemp,
     >        pcm, ecm1, ecm2, bdir, gamma, phi, theta )
         pini= ( ecm1 +ecm2 )/2.d0
         p0(1,1)= 0.d0
         p0(2,1)= 0.d0
         p0(3,1)= pini
         p0(4,1)= pini
         p0(1,2)= 0.d0
         p0(2,2)= 0.d0
         p0(3,2)= -pini
         p0(4,2)= pini
         j= 2
         do i=3,nextrn
            if( i.ne.irad ) then
               j= j +1
               call bstfw( ppart(1,i), bdir, 
     >              gamma, phi, theta, p0(1,j) )
            endif
         enddo
         call setscale( 600, p0, qren, qfac )
         if( qsq.lt.qfac**2 ) return
         iret= 0
      else
         do imoth=1,2
            do i=1,4
               ptemp(i)= ppart(i,imoth) -ppart(i,irad)
            enddo
            qsq= ptemp(1)**2 +ptemp(2)**2 +ptemp(3)**2 -ptemp(4)**2
            if( imoth.eq.1 ) then
               call bstcm( ptemp, ppart(1,2),
     >              pcm, ecm1, ecm2, bdir, gamma, phi, theta )
            else
               call bstcm( ppart(1,1), ptemp,
     >              pcm, ecm1, ecm2, bdir, gamma, phi, theta )
            endif
            pini= ( ecm1 +ecm2 )/2.d0
            p0(1,1)= 0.d0
            p0(2,1)= 0.d0
            p0(3,1)= pini
            p0(4,1)= pini
            p0(1,2)= 0.d0
            p0(2,2)= 0.d0
            p0(3,2)= -pini
            p0(4,2)= pini
            j= 2
            do i=3,nextrn
               if( i.ne.irad ) then
                  j= j +1
                  call bstfw( ppart(1,i), 
     >                 bdir, gamma, phi, theta, p0(1,j) )
               endif
            enddo
            call setscale( 600, p0, qren, qfac )
            if( qsq.lt.qfac**2 ) return
         enddo
         iret= 0
      endif

      return
      end
C--------+---------+---------+---------+---------+---------+---------+
      subroutine sgcval_aa1j( ppart, idpart, prad, weight )
C Soft-divergence correction to the gluon radiation added to AA + 1 jet 
C events.
C       ppart(j,i): 4-momenta of the particles in the non-rad process
C       idpart(i): PDG ID of the particles
C       prad(j): 4-momentum of the gluon radiation
C       weight: eveluated weight
C Note that the momenta must be defined in the CM frame of the non-rad event 
C in which the initial-particle momenta are aligned along the z axis.
C The color charge is set according to the definition in "setcmat_aa2j" 
C in lll_aa2j.f.
      implicit real*8(a-h,o-z)
      real*8 ppart(4,10),prad(4),psum(4)
      integer idpart(10)
      parameter ( Cf= 4.d0/3.d0, Ca= 3.d0 )

      weight= 1.d0
      fcoll= 0.d0
      fcomb= 0.d0

      if( idpart(1).eq.21 .or. idpart(2).eq.21 ) then
         iproc= 601  ! q g -> AA q
      else
         iproc= 602  ! q qbar -> AA g
      endif

      do i=1,4
         psum(i)= 0.d0
         do ipart=3,5
            psum(i)= psum(i) +ppart(i,ipart)
         enddo
      enddo
      shat0= psum(4)**2 -psum(3)**2 -psum(2)**2 -psum(1)**2
      do i=1,4
         psum(i)= psum(i) +prad(i)
      enddo
      shat= psum(4)**2 -psum(3)**2 -psum(2)**2 -psum(1)**2

      if( iproc.eq.601 ) then
C  Initial state radiation.
         call setscale( iproc, ppart, qren, qfac )
         do iemit=1,2
            if( iemit.eq.1 ) then
               ispec1= 2
               ispec2= 5
            else
               ispec1= 1
               ispec2= 5
            endif
            qsq= ( ppart(1,iemit) -prad(1) )**2
     >           +( ppart(2,iemit) -prad(2) )**2
     >           +( ppart(3,iemit) -prad(3) )**2
     >           -( ppart(4,iemit) -prad(4) )**2
            if( qsq.lt.qfac**2 ) then
               zbr= shat0/shat
C  The particle 1 is a quark and 2 is a gluon for the ME calculation of 
C  the process 603.
               if( idpart(iemit).ne.21 ) then
                  fc= Cf*( 1. +zbr**2 )/( 1. -zbr )/qsq
                  cmtx1= Ca/2.d0        ! cmat603(1,2)
                  cmtx2= Cf -Ca/2.d0    ! cmat603(1,5)
                  fs= cmtx1*
     >                 fsgkin( prad, ppart(1,iemit), ppart(1,ispec1) )
     >               +cmtx2*
     >                 fsgkin( prad, ppart(1,iemit), ppart(1,ispec2) )
                  fs= fs*( 1. +zbr**2 )/2.d0
               else
                  fc= 2.*Ca*( 1. -zbr*( 1. -zbr ) )**2
     >                 /zbr/( 1. -zbr )/qsq
                  cmtx1= Ca/2.d0        ! cmat603(2,1)
                  cmtx2= Ca/2.d0        ! cmat603(2,5)
                  fs= cmtx1*
     >                 fsgkin( prad, ppart(1,iemit), ppart(1,ispec1) )
     >               +cmtx2*
     >                 fsgkin( prad, ppart(1,iemit), ppart(1,ispec2) )
                  fs= fs*( 1. -zbr*( 1. -zbr ) )**2/zbr
               endif
               ffcorr= 1.d0/zbr
               fcoll= fcoll +fc*ffcorr
               rsoft= zbr
               fcomb= fcomb +( ( 1. -rsoft )*fc +rsoft*fs )*ffcorr
            endif
         enddo
C  Final state radiation.
         call scale_fsub( iproc, ppart, qmax )
         iemit= 5
         ispec1= 1
         ispec2= 2
         do i=1,4
            psum(i)= ppart(i,iemit) +prad(i)
         enddo
         qsq= psum(4)**2 -psum(3)**2 -psum(2)**2 -psum(1)**2
         if( qsq.le.qmax**2 ) then
            psumabs= sqrt( psum(1)**2 +psum(2)**2 +psum(3)**2 )
            plrad= ( psum(1)*prad(1) +psum(2)*prad(2) +psum(3)*prad(3) )
     >           /psumabs
            zbr= 1.d0 -( prad(4) +plrad )/( psum(4) +psumabs )
            if( plrad.gt.0.d0 .and. plrad.lt.psumabs ) then
               fc= Cf*( 1. +zbr**2 )/( 1. -zbr )/qsq
            else
               fc= 0.d0
            endif
            if( idpart(ispec1).ne.21 ) then
               cmtx1= Cf -Ca/2.d0        ! cmat603(5,1)
               cmtx2= Ca/2.d0            ! cmat603(5,2)
            else
               cmtx1= Ca/2.d0            ! cmat603(5,2)
               cmtx2= Cf -Ca/2.d0        ! cmat603(5,1)
            endif
            fs= cmtx1*fsgkin( prad, ppart(1,iemit), ppart(1,ispec1) )
     >         +cmtx2*fsgkin( prad, ppart(1,iemit), ppart(1,ispec2) )
            fs= fs*( 1. +zbr**2 )/2.d0/zbr
            ffcorr= shat/shat0
            fcoll= fcoll +fc*ffcorr
            rsoft= zbr
            fcomb= fcomb +( ( 1. -rsoft )*fc +rsoft*fs )*ffcorr
         endif
      elseif( iproc.eq.602 ) then
C  Initial state radiation.
         call setscale( iproc, ppart, qren, qfac )
         do iemit=1,2
            if( iemit.eq.1 ) then
               ispec1= 2
               ispec2= 5
            else
               ispec1= 1
               ispec2= 5
            endif
            qsq= ( ppart(1,iemit) -prad(1) )**2
     >           +( ppart(2,iemit) -prad(2) )**2
     >           +( ppart(3,iemit) -prad(3) )**2
     >           -( ppart(4,iemit) -prad(4) )**2
            if( qsq.lt.qfac**2 ) then
               zbr= shat0/shat
               fc= Cf*( 1. +zbr**2 )/( 1. -zbr )/qsq
C  The particle 1 is a quark and 2 is an anti-quark for the ME calculation of 
C  the process 605.
C  Hence, no difference between them concerning the color carge.
               cmtx1= Cf -Ca/2.d0   ! cmat605(1,2) or cmat614(2,1)
               cmtx2= Ca/2.d0       ! cmat605(1,5) or cmat614(2,5)
               fs= cmtx1*
     >              fsgkin( prad, ppart(1,iemit), ppart(1,ispec1) )
     >            +cmtx2*
     >              fsgkin( prad, ppart(1,iemit), ppart(1,ispec2) )
               fs= fs*( 1. +zbr**2 )/2.d0
               ffcorr= 1.d0/zbr
               fcoll= fcoll +fc*ffcorr
               rsoft= zbr
               fcomb= fcomb +( ( 1. -rsoft )*fc +rsoft*fs )*ffcorr
            endif
         enddo
C  Final state radiation.
         call scale_fsub( iproc, ppart, qmax )
         iemit= 5
         ispec1= 1
         ispec2= 2
         do i=1,4
            psum(i)= ppart(i,iemit) +prad(i)
         enddo
         qsq= psum(4)**2 -psum(3)**2 -psum(2)**2 -psum(1)**2
         if( qsq.le.qmax**2 ) then
            psumabs= sqrt( psum(1)**2 +psum(2)**2 +psum(3)**2 )
            plrad= ( psum(1)*prad(1) +psum(2)*prad(2) +psum(3)*prad(3) )
     >           /psumabs
            zbr= 1.d0 -( prad(4) +plrad )/( psum(4) +psumabs )
            if( plrad.gt.0.d0 .and. plrad.lt.psumabs ) then
               fc= 2.*Ca*( 1. -zbr*( 1. -zbr ) )**2/zbr/( 1. -zbr )/qsq
            else
               fc= 0.d0
            endif
            cmtx1= Ca/2.d0        ! cmat605(5,1) or cmat605(5,2)
            cmtx2= Ca/2.d0        ! cmat605(5,2) or cmat605(5,1)
            fs= cmtx1*fsgkin( prad, ppart(1,iemit), ppart(1,ispec1) )
     >         +cmtx2*fsgkin( prad, ppart(1,iemit), ppart(1,ispec2) )
            fs= fs*( 1. -zbr*( 1. -zbr ) )**2/zbr**2
            ffcorr= shat/shat0
            fcoll= fcoll +fc*ffcorr
            rsoft= zbr
            fcomb= fcomb +( ( 1. -rsoft )*fc +rsoft*fs )*ffcorr
         endif
      endif

      if( fcoll.gt.0.d0 ) weight= fcomb/fcoll

      return
      end
