      subroutine lll_aa2j( isub, result, iret )
C  Calculate the matrix element in the LLL (Limited Leading-Log) 
C  approximation.
C  For gamma gamma + 2 jets production processes.
      implicit real*8(a-h,o-z)
      include 'inclk.h'
      include 'grchad.inc'

      parameter ( nextrn= 6, ipj1= 3, ipj2= 6 )
      parameter ( nextrn0= nextrn -1 )
      integer kfcode1(nextrn),kcharg1(nextrn)
      integer kfme(nextrn),kf0(nextrn0),kfme0(nextrn0),imap(nextrn0)
      real*8 bdir(3),ptemp(4),p0(4,nextrn0)
      parameter ( Cf= 4.d0/3.d0, Ca= 3.d0, Tr= 0.5d0 )

      result= 0.d0
      iret= 0

C  Check the process number.
      if( isub.lt.603 .or. isub.gt.609 ) then
         print *,'lll_aa2j: irrelevant process number ',isub
         stop
      endif

C  Save the particle information of the hard interaction, because 
C  it is replaced in the calculation of non-radiative process.
C  It would be enough to save/restore "kfcode" since "amass1/2" are 
C  recalculated when the LHA common is filled.
      do i=1,nextrn
         kfcode1(i)= kfcode(i)
         kcharg1(i)= kcharg(i)
      enddo

C  Original kfcode corresponding to "pgrc", for which the ME is evaluated.
      call izsdet( IZSYSM )      ! in grcpygen.F.
      if( IZSYSM.eq.0 ) then
         kfme(1)= kfcode1(1)
         kfme(2)= kfcode1(2)
      else
         kfme(1)= kfcode1(2)
         kfme(2)= kfcode1(1)
      endif
      do i=3,nextrn
         kfme(i)= kfcode1(i)
      enddo

C  Squared cm energy of this event.
C  The input event must be in the cm frame with initial-state particles 
C  aligned along the z axis.
      stot= ( pgrc(4,1) +pgrc(4,2) )**2

C  === Initial-state radiation ===

C  Loop over jets.
      do irad=ipj1,ipj2
         do imoth=1,2

C  Possible branch mode.
C  No QED branch is considered in the initial-state radiation.
            ibranch= 0
            if( kfme(irad).eq.21 ) then
               if( kfme(imoth).eq.21 ) then
                  ibranch= 2      ! g -> g* g
                  idtemp= 21
               elseif( iabs(kfme(imoth)).lt.10 ) then
                  ibranch= 1      ! q -> q* g
                  idtemp= kfme(imoth)
               endif
            elseif( iabs(kfme(irad)).lt.10 ) then
               if( kfme(imoth).eq.21 ) then
                  ibranch= 3      ! g -> qbar* q
                  idtemp= -kfme(irad)
               elseif( iabs(kfme(imoth)).lt.10 ) then
                  if( kfme(irad).eq.kfme(imoth) ) then
                     ibranch= 4   ! q -> g* q
                     idtemp= 21
                  endif
               endif
            endif
            if( ibranch.lt.1 ) goto 1

C  Particle code of the non-rad event.
            if( imoth.eq.1 ) then
               kf0(1)= idtemp
               kf0(2)= kfme(2)
            else
               kf0(1)= kfme(1)
               kf0(2)= idtemp
            endif
            j= 2
            do i=3,nextrn
               if( i.ne.irad ) then
                  j= j+1
                  kf0(j)= kfme(i)
               endif
            enddo

C  Mapping to a non-radiative process.
            do i=1,nextrn0
               imap(i)= i
            enddo
            call mapnr_aa2j( isub, kfme, irad, imoth, nonrad, imap )
            if( nonrad.lt.1 ) goto 1

            itrans= 0
            if( imap(1).ne.1 ) itrans= 1

C  Particle code of the non-rad event to be used for the ME calculation.
            do i=1,nextrn0
               kfme0(imap(i))= kf0(i)
            enddo

C  Momentum transfer.
            do i=1,4
               ptemp(i)= pgrc(i,imoth) -pgrc(i,irad)
            enddo
            qsq= ptemp(1)**2 +ptemp(2)**2 +ptemp(3)**2 -ptemp(4)**2

C  Boost to the cm frame of the non-radiation subsystem.
            if( imoth.eq.1 ) then
               call bstcm( ptemp, pgrc(1,2),
     >              pcm, ecm1, ecm2, bdir, gamma, phi, theta )
            else
               call bstcm( pgrc(1,1), ptemp,
     >              pcm, ecm1, ecm2, bdir, gamma, phi, theta )
            endif

C  Squared cm energy of the non-radiative subsystem.
C  The branch parameter is defined by its fraction.
            s0= ( ecm1 +ecm2 )**2
            zbr= s0/stot

C  The initial-state particles must be massless in this approximation.
            pini= sqrt( s0 )/2.d0
C  Set the four-momenta in the subsystem's cm frame.
            p0(1,1)= 0.d0
            p0(2,1)= 0.d0
            p0(3,1)= pini
            p0(4,1)= pini
            p0(1,2)= 0.d0
            p0(2,2)= 0.d0
            p0(3,2)= -pini
            p0(4,2)= pini

C  Boost the final-state particles to the CM frame.
C  The direction is reversed if itrans = 1.
C  The particle ordering is corrected.
            j= 2
            do i=3,nextrn
               if( i.ne.irad ) then
                  j= j +1
                  k= imap(j)
                  call bstfw( pgrc(1,i), 
     >                 bdir, gamma, phi, theta, p0(1,k) )
                  if( itrans.eq.1 ) p0(3,k)= -p0(3,k)
               endif
            enddo

C  No approximation if the radiation is harder than the factorization scale.
            call setscale( nonrad, p0, qren, qfac )

            if( qsq.lt.qfac**2 ) then

C  Matrix element of the non-radiative process.
               iret= 1
               call menr_aa2j( nonrad, kfme0, p0, result0, iret )
               if( iret.ne.0 ) goto 9

C  Radiation factor.
               factor= 0.d0
               if( ibranch.eq.1 ) then       ! q -> q* g
                  factor= Cf*( 1 +zbr**2 )/( 1 -zbr )/qsq
               elseif( ibranch.eq.2 ) then   ! g -> g* g
                  factor= 2.d0*Ca*(1-zbr*(1-zbr))**2/zbr/(1-zbr)/qsq
               elseif( ibranch.eq.3 ) then   ! g -> qbar* q
                  factor= Tr*( zbr**2 +( 1 -zbr )**2 )/qsq
               elseif( ibranch.eq.4 ) then   ! q -> g* q
                  factor= Cf*( 1 +( 1 -zbr )**2 )/zbr/qsq
               endif

C  Soft-gluon singularity.
C  "fsgkin" can be found in grckorg.F.
               if( kfme(irad).eq.21 ) then
                  iemit= imoth
                  fsoft= 0.d0
                  do ispec=1,2
                     if( ispec.ne.iemit ) then
                        cchrg= cchrg_aa2j( isub, iemit, ispec )
                        fsoft= fsoft +cchrg*fsgkin( pgrc(1,irad),
     >                       pgrc(1,iemit), pgrc(1,ispec))
                     endif
                  enddo
                  do ispec=ipj1,ipj2
                     if( kfme(ispec).ne.22 .and. ispec.ne.irad ) then
                        cchrg= cchrg_aa2j( isub, iemit, ispec )
                        fsoft= fsoft +cchrg*fsgkin( pgrc(1,irad),
     >                       pgrc(1,iemit), pgrc(1,ispec))
                     endif
                  enddo

C  Correction to match the collinear approximation.
                  if( ibranch.eq.1 ) then
                     fsoft= fsoft*( 1.d0 +zbr**2 )/2.d0
                  elseif( ibranch.eq.2 ) then
                     fsoft= fsoft*( 1.d0 -zbr*( 1.d0 -zbr ) )**2/zbr
                  else
                     fsoft= 0.d0  ! for safety
                  endif

C  Combined radiation factor.
                  rsoft= zbr
                  factor= ( 1.d0 -rsoft )*factor +rsoft*fsoft
               endif

C  Matrix element in the collinear-soft combined approximation.
               ffcorr= 1.d0/zbr
               result= result +8.d0*pi*alphas*result0*factor*ffcorr

            endif
 1          continue
         enddo
      enddo

C  === Final-state radiation ===

C  Loop over all combinations.
      do ip1=ipj1,ipj2-1
         do ip2=ip1+1,ipj2

C  Possible branch mode.
C    radiation = less energetic gluon for g* -> g g
C    radiation = gluon for q* -> q g
C    according to the particle ordering for g* -> q qbar
C    radiation = photon for q* -> q gamma
            ibranch= 0
            if( kfme(ip1).eq.21 ) then
               if( kfme(ip2).eq.21 ) then
                  ibranch= 2      ! g* -> g g
                  if( pgrc(4,ip1).ge.pgrc(4,ip2) ) then
                     ichild= ip1
                     irad= ip2
                  else
                     ichild= ip2
                     irad= ip1
                  endif
                  idtemp= 21
               elseif( iabs(kfme(ip2)).lt.10 ) then
                  ibranch= 1      ! q* -> g q
                  ichild= ip2
                  irad= ip1
                  idtemp= kfme(ip2)
               endif
            elseif( iabs(kfme(ip1)).lt.10 ) then
               if( kfme(ip2).eq.21 ) then
                  ibranch= 1      ! q* -> q g
                  ichild= ip1
                  irad= ip2
                  idtemp= kfme(ip1)
               elseif( iabs(kfme(ip2)).lt.10 ) then
                  if( kfme(ip1)+kfme(ip2).eq.0 ) then
                     ibranch= 3   ! g* -> q qbar
                     ichild= ip1
                     irad= ip2
                     idtemp= 21
                  endif
               elseif( kfme(ip2).eq.22 ) then
                  ibranch= 5      ! q* -> q gamma
                  ichild= ip1
                  irad= ip2
                  idtemp= kfme(ip1)
               endif
            elseif( kfme(ip1).eq.22 ) then  ! photon
               if( iabs(kfme(ip2)).lt.10 ) then
                  ibranch= 5      ! q* -> gamma q
                  ichild= ip2
                  irad= ip1
                  idtemp= kfme(ip2)
               endif
            endif
            if( ibranch.lt.1 ) goto 2

C  Particle code of the non-rad event.
            j= 0
            do i=1,nextrn
               if( i.ne.irad ) then
                  j= j+1
                  if( i.eq.ichild ) then
                     kf0(j)= idtemp
                     imoth= j   ! mother in the non-rad event
                  else   
                     kf0(j)= kfme(i)
                  endif
               endif
            enddo

C  Mapping to a non-radiative process.
            do i=1,nextrn0
               imap(i)= i
            enddo
            call mapnr_aa2j( isub, kfme, irad, ichild, nonrad, imap )
            if( nonrad.lt.1 ) goto 2

C  Particle code of the non-rad event to be used for the ME calculation.
            do i=1,nextrn0
               kfme0(imap(i))= kf0(i)
            enddo

C  Momentum sum.
            do i=1,4
               ptemp(i)= pgrc(i,ichild) +pgrc(i,irad)
            enddo

C  Parameters of the considered branch.
C  "zbr" is defined to be the momentum fraction in an infinite-momentum frame.
            psum= sqrt( ptemp(1)**2 +ptemp(2)**2 +ptemp(3)**2 )
            esum= ptemp(4)
            qsq= esum**2 -psum**2
            pl= ( pgrc(1,ichild)*ptemp(1) +pgrc(2,ichild)*ptemp(2)
     >           +pgrc(3,ichild)*ptemp(3) )/psum
            zbr= ( pl*( esum +psum ) +qsq/2. )
     >           /( psum*( esum +psum ) +qsq )

C  Set the four-momenta in the subsystem's cm frame.
C  The momentum adjustment is not applied to the final state.
C  Instead, the initial-state momenta are changed to conserve the energy.
            etot= psum
            do ip=3,nextrn
               if( ip.ne.ichild .and. ip.ne.irad ) then
                  etot= etot +pgrc(4,ip)
               endif
            enddo
            pini= etot/2.d0
            p0(1,1)= 0.d0
            p0(2,1)= 0.d0
            p0(3,1)= pini
            p0(4,1)= pini
            p0(1,2)= 0.d0
            p0(2,2)= 0.d0
            p0(3,2)= -pini
            p0(4,2)= pini

C  Final-state particles.
            jp= 2
            do ip=3,nextrn
               if( ip.ne.irad ) then
                  jp= jp +1
                  kp= imap(jp)
                  if( ip.eq.ichild ) then
                     do i=1,3
                        p0(i,kp)= ptemp(i)
                     enddo
                     p0(4,kp)= psum
                  else
                     do i=1,4
                        p0(i,kp)= pgrc(i,ip)
                     enddo
                  endif
               endif
            enddo

C  No approximation if the radiation is harder than the factorization scale.
            call scale_fsub( nonrad, p0, qmax )
            if( qsq.lt.qmax**2 ) then

C  Matrix element of the non-radiative process.
               iret= 1
               call menr_aa2j( nonrad, kfme0, p0, result0, iret )
               if( iret.ne.0 ) goto 9

C  Radiation factor.
C  Requrement of a forward radiation because QCDPSf does not emit 
C  backward radiations.
               if( pl.gt.0.d0 .and. pl.lt.psum ) then
                  if( ibranch.eq.1 ) then     ! q -> q* g
                     factor= Cf*( 1 +zbr**2 )/( 1 -zbr )/qsq
                  elseif( ibranch.eq.2 ) then ! g -> g* g
                     factor= 2.d0*Ca*(1-zbr*(1-zbr))**2/zbr/(1-zbr)/qsq
                  elseif( ibranch.eq.3 ) then ! g -> qbar* q
                     factor= Tr*( zbr**2 +( 1 -zbr )**2 )/qsq
                  elseif( ibranch.eq.5 ) then ! q -> q gamma
                     if( mod( iabs( kfme(ichild) ), 2 ).eq.1 ) then
                        cqed= 1.d0/9.d0
                     else
                        cqed= 4.d0/9.d0
                     endif
                     factor= alpha/alphas*
     >                    cqed*( 1 +zbr**2 )/( 1 -zbr )/qsq
                  endif
               else
                  factor= 0.d0
               endif

C  Soft-gluon singularity.
               if( kfme(irad).eq.21 ) then
                  iemit= ichild
                  fsoft= 0.d0
                  do ispec=1,2
                     cchrg= cchrg_aa2j( isub, iemit, ispec )
                     fsoft= fsoft +cchrg*fsgkin( pgrc(1,irad),
     >                    pgrc(1,iemit), pgrc(1,ispec))
                  enddo
                  do ispec=ipj1,ipj2
                     if( kfme(ispec).ne.22 .and.
     >                    ispec.ne.irad .and. ispec.ne.iemit ) then
                        cchrg= cchrg_aa2j( isub, iemit, ispec )
                        fsoft= fsoft +cchrg*fsgkin( pgrc(1,irad),
     >                       pgrc(1,iemit), pgrc(1,ispec))
                     endif
                  enddo

C  Correction to match the collinear approximation.
                  if( ibranch.eq.1 ) then
                     fsoft= fsoft*( 1.d0 +zbr**2 )/2.d0/zbr
                  elseif( ibranch.eq.2 ) then
                     fsoft= fsoft*( 1.d0 -zbr*( 1.d0 -zbr ) )**2/zbr**2
                  else
                     fsoft= 0.d0  ! for safety
                  endif

C  Combined radiation factor.
                  rsoft= zbr
                  factor= ( 1.d0 -rsoft )*factor +rsoft*fsoft
               endif

C  Matrix element in the collinear-soft combined approximation.
               s0= ( 2.d0*pini )**2
               ffcorr= stot/s0
               result= result +8.d0*pi*alphas*result0*factor*ffcorr

            endif
 2          continue
         enddo
      enddo

 9    continue
C  Restore the saved variables.
      do i=1,nextrn
         kfcode(i)= kfcode1(i)
         kcharg(i)= kcharg1(i)
      enddo

      return
      end
C----------------------------------------------------------------------------
      subroutine mapnr_aa2j( isub, kf, irad, iemit, nonrad, imap )
C  Mapping to a non-radiative process.
C  The particle ordering may have to be corrected in the mapped event; 
C  imap(i) is used for the correction.
      parameter ( nextrn= 6, nextrn0= nextrn -1 )
      integer kf(nextrn),imap(nextrn0)

      nonrad= 0
      itrans= 0

      if( isub.eq.603 ) then         ! u g -> AA u g
         if( irad.eq.3 .or. irad.eq.4 ) then
            nonrad= 612
         elseif( irad.eq.5 ) then
            if( iemit.eq.2 ) then
               nonrad= 602
            endif
         elseif( irad.eq.6 ) then
            nonrad= 601
         endif
      elseif( isub.eq.604 ) then         ! u ubar -> AA u ubar
         if( irad.eq.3 .or. irad.eq.4 ) then
            nonrad= 613
         elseif( irad.eq.5 .or. irad.eq.6 ) then
            if( iemit.le.2 ) then
               nonrad= 601
               if( iemit.eq.1 ) itrans= 1
            else
               nonrad= 602
            endif
         endif
      elseif( isub.eq.605 ) then         ! u ubar -> AA g g
         if( irad.eq.3 .or. irad.eq.4 ) then
            nonrad= 614
         elseif( irad.eq.5 .or. irad.eq.6 ) then
            nonrad= 602
         endif
      elseif( isub.eq.606 ) then         ! g g -> AA u ubar
         if( irad.eq.3 .or. irad.eq.4 ) then
            nonrad= 615
         elseif( irad.eq.5 .or. irad.eq.6 ) then
            if( iemit.le.2 ) then
               nonrad= 601
               if( iemit.eq.2 ) itrans= 1
            endif
         endif
      elseif( isub.eq.607 ) then         ! u dbar -> AA u dbar
         if( irad.eq.3 .or. irad.eq.4 ) then
            nonrad= 616
         elseif( irad.eq.5 .or. irad.eq.6 ) then
            nonrad= 601
            if( iemit.eq.1 ) itrans= 1
         endif
      elseif( isub.eq.608 ) then         ! u u -> AA u u
         if( irad.eq.3 .or. irad.eq.4 ) then
            nonrad= 617
         elseif( irad.eq.5 .or. irad.eq.6 ) then
            nonrad= 601
            if( iemit.eq.1 ) itrans= 1
         endif
      elseif( isub.eq.609 ) then         ! u d -> A u d
         if( irad.eq.3 .or. irad.eq.4 ) then
            nonrad= 618
         elseif( irad.eq.5 .or. irad.eq.6 ) then
            nonrad= 601
            if( iemit.eq.1 ) itrans= 1
         endif
      else
         print *,'mapnr_aa2j: process ',isub,' not supported.'
         stop
      endif

      if( itrans.eq.1 ) then
         imap(1)= 2
         imap(2)= 1
      endif

      return
      end
C----------------------------------------------------------------------------
      subroutine menr_aa2j( nonrad, kfme0, p0, result0, iret )
      implicit real*8(a-h,o-z)
      parameter ( nextrn0= 5 )
      integer kfme0(nextrn0)
      real*8 p0(4,nextrn0)

      iret= 0
      if( nonrad.eq.601 ) then
         call me601( kfme0, p0, result0, iret )
      elseif( nonrad.eq.602 ) then
         call me602( kfme0, p0, result0, iret )
      elseif( nonrad.eq.612 ) then
         call me612( kfme0, p0, result0, iret )
      elseif( nonrad.eq.613 ) then
         call me613( kfme0, p0, result0, iret )
      elseif( nonrad.eq.614 ) then
         call me614( kfme0, p0, result0, iret )
      elseif( nonrad.eq.615 ) then
         call me615( kfme0, p0, result0, iret )
      elseif( nonrad.eq.616 ) then
         call me616( kfme0, p0, result0, iret )
      elseif( nonrad.eq.617 ) then
         call me617( kfme0, p0, result0, iret )
      elseif( nonrad.eq.618 ) then
         call me618( kfme0, p0, result0, iret )
      else
         print *,'menr_aa2j: illegal process number ',nonrad
         stop
      endif

      return
      end
C----------------------------------------------------------------------------
      double precision function cchrg_aa2j( isub, iemit, ispec )
C  Return the color charge for the soft-gluon approximation.
      implicit real*8(a-h,o-z)
      parameter ( nextrn= 6 )
      real*8 cmat603,cmat605
      common /cmat_aa2j/cmat603(nextrn,nextrn),cmat605(nextrn,nextrn)

      logical first/.true./

      cchrg_aa2j= 0.d0

      if( first ) then
         call setcmat_aa2j
         first= .false.
      endif

      if( isub.eq.603 ) then
         cchrg_aa2j= cmat603(iemit,ispec)
      elseif( isub.eq.605 ) then
         cchrg_aa2j= cmat605(iemit,ispec)
      else
         print *,'cchrg_aa2j: process ',isub,' not supported'
         stop
      endif

      return
      end
C----------------------------------------------------------------------------
      subroutine setcmat_aa2j
      implicit real*8(a-h,o-z)
      parameter ( nextrn= 6 )
      real*8 cmat603,cmat605
      common /cmat_aa2j/cmat603(nextrn,nextrn),cmat605(nextrn,nextrn)
      parameter ( Cf= 4.d0/3.d0, Ca= 3.d0 )

      do i=1,nextrn
         do j=1,nextrn
            cmat603(i,j)= 0.d0
            cmat605(i,j)= 0.d0
         enddo
      enddo

      cmat603(1,2)= Ca/2.d0
      cmat603(1,5)= Cf -Ca/2.d0
      cmat603(2,5)= Ca/2.d0
      cmat603(2,1)= cmat603(1,2)
      cmat603(5,1)= cmat603(1,5)
      cmat603(5,2)= cmat603(2,5)

      cmat605(1,2)= Cf -Ca/2.d0
      cmat605(1,5)= Ca/2.d0
      cmat605(1,6)= Ca/2.d0
      cmat605(2,5)= Ca/2.d0
      cmat605(2,6)= Ca/2.d0
      cmat605(2,1)= cmat605(1,2)
      cmat605(5,1)= cmat605(1,5)
      cmat605(5,2)= cmat605(2,5)
      cmat605(6,1)= cmat605(1,6)
      cmat605(6,2)= cmat605(2,6)

      return
      end
