      SUBROUTINE UPINIT
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)
      INCLUDE 'grchad.inc'

C...User process initialization commonblock.
      INTEGER MAXPUP
      PARAMETER (MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON/HEPRUP/IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),XMAXUP(MAXPUP),
     &LPRUP(MAXPUP)
      SAVE /HEPRUP/

      INTEGER I,J
      CHARACTER*3 CBEAM
      DOUBLE PRECISION SIGMAX

#include "define.h"
#if PDFLIB == 2
C...PDFLIB parameters.
      CHARACTER*20      PARM(20)
      DOUBLE PRECISION  VALUE(20)
#endif

C...Initialization of GR@PPA parameters.
      CALL GRCINIT
C-----------------------------------------------------------------------
C  Beams: 'PP' for proton-proton, and 'PAP' for proton-antiproton.
C-----------------------------------------------------------------------
      CBEAM = 'PP'              ! LHC
C      CBEAM = 'PAP'             ! Tevatron
C-----------------------------------------------------------------------
C  CM energy in GeV
C-----------------------------------------------------------------------
      GRCECM = 14000.0D0         ! LHC
C      GRCECM =  1960.0D0         ! Tevatron RunII
C-----------------------------------------------------------------------
C  Number of external processes and process numbers. 
C-----------------------------------------------------------------------
      NPRUP = 2
      LPRUP(1) = 150            ! AA + 0 jet
      LPRUP(2) = 151            ! AA + 1 jet
C-----------------------------------------------------------------------
C  Jet matching; 0 for no matching (default)
C-----------------------------------------------------------------------
      matching= 1  ! LLL subtraction
C-----------------------------------------------------------------------
C  PS implementation; 0 for turn off (default)
C-----------------------------------------------------------------------
C  Initial state
C      ishower= 1   ! Kurihara's QCDPS for the initial state
C      ishower= 2   ! Backward evolution by QCDPSb
C      ishower= 3   ! QCDPS + minor branches
      ishower= 4   ! QCDPSb + minor branches
C-----------------------------------------------------------------------
C  Final state
      ishwfin= 1   !QCDPSf for the final state
C-----------------------------------------------------------------------
C  Cuts for gammas: pt, eta, and Delta-R
C-----------------------------------------------------------------------
      ptcut= 15.0d0
      etacut= 3.0d0
      drcut= 0.01d0

      GRCPTCUT(1)   = ptcut
      GRCETACUT(1)  = etacut
      GRCRCONCUT(1) = drcut
      GRCPTCUT(2)   = ptcut
      GRCETACUT(2)  = etacut
      GRCRCONCUT(2) = drcut
C-----------------------------------------------------------------------
C  Jet flavor to be produced (1/0:on/off)
C-----------------------------------------------------------------------
      IGJFLV(1) = 1             ! d-jet
      IGJFLV(2) = 1             ! u-jet
      IGJFLV(3) = 1             ! s-jet
      IGJFLV(4) = 1             ! c-jet
      IGJFLV(5) = 1             ! b-jet
      IGJFLV(7) = 1             ! gluon-jet
C-----------------------------------------------------------------------
C  Initial seed of random numbers
C-----------------------------------------------------------------------
      IRSEED = 1234567
C-----------------------------------------------------------------------
C  PDF choice
C-----------------------------------------------------------------------
C  For safety, ...
      PDFGUP(1) = -1
      PDFSUP(1) = -1
      PDFGUP(2) = PDFGUP(1)
      PDFSUP(2) = PDFSUP(1)
C-----------------------------------------------------------------------
#if PDFLIB == 2
C  CERN PDFLIB
      PARM(1) = 'Init0'
      VALUE(1) = 0.0d0
      CALL PDFSET(PARM, VALUE)
C  CTEQ5L
      PARM(1) = 'Nptype'
      VALUE(1) = 1
      PARM(2) = 'Ngroup'
      VALUE(2) = 4
      PARM(3) = 'Nset'
      VALUE(3) = 46
      CALL PDFSET(PARM, VALUE)

      PDFGUP(1) = VALUE(2)
      PDFSUP(1) = VALUE(3)
      PDFGUP(2) = PDFGUP(1)
      PDFSUP(2) = PDFSUP(1)
C-----------------------------------------------------------------------
#elif PDFLIB == 3
C  LHAPDF: a symbolic link "PDFsets" is created during the setup.
      CALL InitPDFset('PDFsets/MRST2007lomod.LHgrid')
      CALL InitPDF(0)
C-----------------------------------------------------------------------
#elif PDFLIB == 4
C  Built-in CTEQ6L1
      CALL SetCtq6(4)
C-----------------------------------------------------------------------
#endif

C======= Do not touch the following lines. ===========

C...Kinematical cuts for "jets".
      GPTCUT   = 1.0D0          ! Pt Cut
      GETACUT  = 10.0D0         ! Eta Cut
      GRCONCUT = drcut          ! RCone Cut

C...Initialization of QCDPS.
      if( ishower+ishwfin.gt.0 ) then
         Ebeam= GRCECM/2.d0
         call QPINIT(Ebeam)
      endif

C...Set incoming beams in the interface common.
      IF (CBEAM.EQ.'PP') THEN
         IDBMUP(1) = 2212
         IDBMUP(2) = 2212
      ELSEIF (CBEAM.EQ.'PAP') THEN
         IDBMUP(1) = 2212
         IDBMUP(2) = -2212
      ELSE
         PRINT '('' Incorrect beam definition: CBEAM = '',A4)',
     >        CBEAM
         STOP
      ENDIF
      EBMUP(1) = GRCECM/2.0D0
      EBMUP(2) = GRCECM/2.0D0

C...Weighting strategy: unweighted on input.
      IDWTUP = -3

C...Check process.
      DO I = 1,NPRUP
         IF (LPRUP(I).LT.100.OR.LPRUP(I).GT.700) THEN
            WRITE(*,*) 'GRC::Error: Wrong process ID',LPRUP(I),'.'
            STOP
         ENDIF
      ENDDO

C...Integration step of GR@PPA.
      DO I = 1,NPRUP
         CALL GRCPYGEN(CBEAM,LPRUP(I),1,SIGMAX)
         XSECUP(i) = SIGMAX
         XMAXUP(i) = 1.0D0
      ENDDO

C...Done.
      RETURN
      END
