C  _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C _/_/  Set User Defined Energy Scale   _/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
      SUBROUTINE GRCUSRSETQ
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'grchad.inc'
      INCLUDE 'inclm.h'
C...User-defined energy scale.
C
C   The definition is effective only when "ICOUP" or "IFACT" = 6.
C   Users must set their-defined energy scale into "GRCQ" or "GRCFAQ".
C
C...4-momentum in the CM frame of the hard interaction
C   PGRC(i,j)  (i=1,4:(px,py,pz,E)): 4-momentum of j-th particle.
C   PQGRC(i,j) (i=1,4:(px,py,pz,E)): 4-momentum of j-th resonance particle.

      call setscale( igsub, pgrc, grcq, grcfaq )

      RETURN
      END
C------------------------------------------------------------------------
      subroutine setscale( igsub, p, qren, qfac )
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'inclm.h'
      real*8 p(4,10)
      parameter ( qmin= 10.d0 )

      qren= qmin
      qfac= qmin

      if( ( igsub.ge.150 .and. igsub.le.151 )
     >     .or. ( igsub.ge.600 .and. igsub.le.602 )
     >     .or. igsub.eq.190
     >     .or. ( igsub.ge.610 .and. igsub.le.611 ) ) then
         qdif= sqrt( ( p(1,3) -p(1,4) )**2 +( p(2,3) -p(2,4) )**2 )/2.d0
         if( qdif.gt.qmin ) then
            qren= qdif
            qfac= qdif
         endif
      endif

      return
      end
C------------------------------------------------------------------------
      subroutine scale_fsr( scale )
C  Return the FSR scale.
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'grchad.inc'

      scale= GRCFAQ

      RETURN
      END
C------------------------------------------------------------------------
      subroutine scale_fsub( igsub, p, scale )
C  Return the energy scale used in the final-state LLL subtraction.
      IMPLICIT REAL*8(A-H,O-Z)
      real*8 p(4,10)

      call setscale( igsub, p, qren, qfac )
      scale= qfac

      RETURN
      END
C  _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C _/_/  Set User Defined Cut  _/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
      SUBROUTINE GRCUSRCUT(IUSRCUT)
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'grchad.inc'
      INTEGER IUSRCUT
      INTEGER GRCKFCD
      EXTERNAL GRCKFCD
      DOUBLE PRECISION GRCIVMS,GRCDPHI,GRCDRCN
      EXTERNAL GRCIVMS,GRCDPHI,GRCDRCN
C...User-defined kinematic cuts.
C
C   The returned argument is "IUSRCUT".
C   If events pass their cuts, IUSRCUT = 0,
C         not pass             IUSRCUT = 1.
C
C   Users must carefully check the convergence of the BASES iteration 
C   if they define a tight cut using this subroutine.
C
C...4-momentum in the CM frame of the hard interaction
C     PGRC(I,J)  (I=1,4:(px,py,pz,E))  : 4-momentum of J-th particle.
C
C...4-momentum in the Laboratory frame.
C     PLGRC(I,J) (I=1,4:(pT,phi,eta,E)): 4-momentum of J-th (J > 2) 
C                                        final state particle.
C
C...Useful stuff.
C     FUNCTION GRCKFCD(I)   : Output particle identification code.
C     FUNCTION GRCIVMS(I,J) : Output inv. mass between "I" and "J" particles.
C     FUNCTION GRCDPHI(I,J) : Output Delta Phi between "I" and "J" particles.
C     FUNCTION GRCDRCN(I,J) : Output Delta R between "I" and "J" particles.
 
      parameter ( ptcut_high= 25.d0, drsepa= 0.3d0 )

C...Initialzation.
      IUSRCUT= 0
 
      if( ( igsub.ge.150 .and. igsub.le.151 )
     >     .or. ( igsub.ge.600 .and. igsub.le.602 )
     >     .or. igsub.eq.190
     >     .or. ( igsub.ge.610 .and. igsub.le.611 ) ) then
         pt1= PLGRC(1,3)
         pt2= PLGRC(1,4)
         if( pt1.ge.ptcut_high .or. pt2.ge.ptcut_high ) then
            phi1= PLGRC(2,3)
            phi2= PLGRC(2,4)
            csdphi= cos( phi1 )*cos( phi2 ) +sin( phi1 )*sin( phi2 )
            if( csdphi.gt.1.d0 ) csdphi= 1.d0
            if( csdphi.lt.-1.d0 ) csdphi= -1.d0
            dphi= acos( csdphi )
            deta= PLGRC(3,3) -PLGRC(3,4)
            drgg= sqrt( dphi**2 +deta**2 )
            if( drgg.lt.drsepa ) IUSRCUT= 1
         else
            IUSRCUT= 1
         endif
      endif

      RETURN
      END
 
C//////////////////////////////////////////
C  Laboratory frame cut inside "FUNC".
C  To be calles when "labcut" = 1.
C//////////////////////////////////////////
      subroutine grclabcut( icut )
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      icut= 0

      return
      end
C//////////////////////////////////////////
C  Process-dependent request of the soft-gluon divergence correction to PS.
C  Effective only when it is activated by settting as isgcorr = 1 in UPINIT.
C//////////////////////////////////////////
      subroutine sgcreq( igsub, ireq )
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)

      ireq= 0

C  aa + 0 jet
      if( igsub.eq.150 .or. igsub.eq.600 ) then
         ireq= 0
C  aa + 1 jet
      elseif( igsub.eq.151 .or. 
     >        ( igsub.ge.601 .and. igsub.le.602 ) ) then
         ireq= 1
C  aa + 2 jets
      elseif( igsub.eq.152 .or. 
     >        ( igsub.ge.603 .and. igsub.le.609 ) ) then
         ireq= 1
C  gg -> aa
      elseif( igsub.eq.155 .or. igsub.eq.700 ) then
         ireq= 0
      endif

      return
      end
