C  _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C _/_/  Set User Defined Energy Scale   _/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
      SUBROUTINE GRCUSRSETQ
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'grchad.inc'
      INCLUDE 'inclm.h'
C...User-defined energy scale.
C
C   The definition is effective only when "ICOUP" or "IFACT" = 6.
C   Users must set their-defined energy scale into "GRCQ" or "GRCFAQ".
C
C...4-momentum in the CM frame of the hard interaction
C   PGRC(i,j)  (i=1,4:(px,py,pz,E)): 4-momentum of j-th particle.
C   PQGRC(i,j) (i=1,4:(px,py,pz,E)): 4-momentum of j-th resonance particle.

      call setscale( igsub, pgrc, grcq, grcfaq )

      RETURN
      END
C------------------------------------------------------------------------
      subroutine setscale( igsub, p, qren, qfac )
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'inclm.h'
      real*8 p(4,10)
      parameter ( qmin= 10.d0 )

      qren= qmin
      qfac= qmin

      if( ( igsub.ge.150 .and. igsub.le.151 )
     >     .or. ( igsub.ge.600 .and. igsub.le.602 )
     >     .or. igsub.eq.190
     >     .or. ( igsub.ge.610 .and. igsub.le.611 ) ) then
         qdif= sqrt( ( p(1,3) -p(1,4) )**2 +( p(2,3) -p(2,4) )**2 )/2.d0
         if( qdif.gt.qmin ) then
            qren= qdif
            qfac= qdif
         endif
      endif

      return
      end
C------------------------------------------------------------------------
      subroutine scale_fsr( scale )
C  Return the FSR scale.
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'grchad.inc'

      scale= GRCFAQ

      RETURN
      END
C------------------------------------------------------------------------
      subroutine scale_fsub( igsub, p, scale )
C  Return the energy scale used in the final-state LLL subtraction.
      IMPLICIT REAL*8(A-H,O-Z)
      real*8 p(4,10)

      call setscale( igsub, p, qren, qfac )
      scale= qfac

      RETURN
      END
C  _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C _/_/  Set User Defined Cut  _/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
      SUBROUTINE GRCUSRCUT(IUSRCUT)
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'grchad.inc'
      INTEGER IUSRCUT
      INTEGER GRCKFCD
      EXTERNAL GRCKFCD
      DOUBLE PRECISION GRCIVMS,GRCDPHI,GRCDRCN
      EXTERNAL GRCIVMS,GRCDPHI,GRCDRCN
C...User-defined kinematic cuts.
C
C   The returned argument is "IUSRCUT".
C   If events pass their cuts, IUSRCUT = 0,
C         not pass             IUSRCUT = 1.
C
C   Users must carefully check the convergence of the BASES iteration 
C   if they define a tight cut using this subroutine.
C
C...4-momentum in the CM frame of the hard interaction
C     PGRC(I,J)  (I=1,4:(px,py,pz,E))  : 4-momentum of J-th particle.
C
C...4-momentum in the Laboratory frame.
C     PLGRC(I,J) (I=1,4:(pT,phi,eta,E)): 4-momentum of J-th (J > 2) 
C                                        final state particle.
C
C...Useful stuff.
C     FUNCTION GRCKFCD(I)   : Output particle identification code.
C     FUNCTION GRCIVMS(I,J) : Output inv. mass between "I" and "J" particles.
C     FUNCTION GRCDPHI(I,J) : Output Delta Phi between "I" and "J" particles.
C     FUNCTION GRCDRCN(I,J) : Output Delta R between "I" and "J" particles.
 
      parameter ( ptcut_high= 30.d0, drsepa= 0.3d0 )

C...Initialzation.
      IUSRCUT= 0

      RETURN
      END
 
C//////////////////////////////////////////
C  Laboratory frame cut inside "FUNC".
C  To be calles when "labcut" = 1.
C//////////////////////////////////////////
      subroutine grclabcut( icut )
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      parameter ( ptcut= 23.d0, etacut= 2.7d0, drcut= 0.3 )

      icut= 1

      ngam0= 0
      ngam= 0
      do i=1,NUP
         if( IDUP(i).eq.22 ) then
            ngam0= ngam0 +1
            ptsq= PUP(1,i)**2 +PUP(2,i)**2
            pt= sqrt( ptsq )
            if( pt.ge.ptcut ) then
               pz= PUP(3,i)
               pabs= sqrt( ptsq +pz**2 )
               etaabs= log( ( pabs +abs( pz ) )/pt )
               if( etaabs.le.etacut ) then
                  ngam= ngam +1
                  eta= etaabs
                  if( pz.lt.0.d0 ) eta= -eta
                  if( ngam.eq.1 ) then
                     ip1= i
                     pt1= pt
                     eta1= eta
                  elseif( ngam.eq.2 ) then
                     ip2= i
                     pt2= pt
                     eta2= eta
                  endif
               endif
            endif
         endif
      enddo
      if( ngam0.ne.2 ) then
         print *,'grclabcut: num. of gammas recorded is not 2 but ',
     >        ngam0
         stop
      endif
      if( ngam.lt.2 ) return

      csphi= ( PUP(1,ip1)*PUP(1,ip2) +PUP(2,ip1)*PUP(2,ip2) )/pt1/pt2
      if( csphi.gt.1.d0 ) csphi= 1.d0
      if( csphi.lt.-1.d0 ) csphi= -1.d0
      dphi= acos( csphi )
      dr= sqrt( dphi**2 +( eta1 -eta2 )**2 )
      if( dr.lt.drcut ) return

      icut= 0

      return
      end
C//////////////////////////////////////////
C  Process-dependent request of the soft-gluon divergence correction to PS.
C  Effective only when it is activated by settting as isgcorr = 1 in UPINIT.
C//////////////////////////////////////////
      subroutine sgcreq( igsub, ireq )
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)

      ireq= 0

C  a + 1 jet
      if( igsub.eq.190 .or. 
     >        ( igsub.ge.610 .and. igsub.le.611 ) ) then
         ireq= 1
C  a + 2 jets
      elseif( igsub.eq.191 .or. 
     >        ( igsub.ge.612 .and. igsub.le.618 ) ) then
         ireq= 1
      endif

      return
      end
