      SUBROUTINE QCDPSf( iaddptn, Escl,  Weight)
C Input
C iaddptn: address of a final parton to be parton showered
C Escl  : Energy scale of PS evolution
C
C Output
C Weight: Weight of this event
C
C 08/Sep./2009   					Y. Kurihara
C
C Modified so that the two branch products are treated equally to generate 
C further branches. Accordingly, another modification was made to determine 
C the splitting parameter every time when a new branch is generated by 
C QPGENf, because we need this parameter to give the maximum energy scale of 
C the subsequent branches.
C 7 May 2010, S. Odaka
C
C A new parameter "epsfin" was added to "qpincl.h" so that we can apply 
C a different cutoff to the space-like and time-like PS.
C 11 May 2010, S.O.
C
C The longitudinal momentum of branch products are now defined in "pskfin" 
C so that the branch parameter "z" should be given as the momentum fraction 
C in the infinite-momentum frame. The "backward" branches that this new 
C definition may produce are rejected in "QCDPSf" beforehand. This condition 
C suppresses the branch of soft branch products, but no clear constraint on 
C the max. value of Q^2. So, no constraint is set in "QPXfin".
C 28 Jun. 2010, S.O.
C
C The QED photon radiation from quarks are supported. The on/off is controled 
C by the parameter "ifqed" in qpincl.h.
C 7 Sep. 2011, S.O.
C
C The method to determine whether a photon should be radiated or not was 
C changed to that of PYSHOW in PYTHIA.
C 2 Nov. 2011, S.O.
C
C A scheme to force QED radiation is supported.
C Relevant routines returns the event weight in each step.
C Dec 14 2011, S.O.
*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      COMMON /QPIF/Iif
      common/QPforce/pphff(5),q2ph(4),ephmin,nphgen,ipmain,idmain,nph1
      common/QPforce1/iph1

* Final state PS (time-like evolution)
      Iif = 1

      Weight= 1.

      Lp  = iaddptn
      NTOP = NTOP + 1
      L   = NTOP
      L0  = L

C  Supplementary information for the parton to evolve.
      NLPTN(3,Lp) = 1
      NLPTN(4,Lp) = L-Lp

      pp= sqrt( PLPTN(4,Lp)**2+PLPTN(5,Lp)**2+PLPTN(6,Lp)**2 )
      Es2= Escl**2

C  Set the information to start the evolution.
      PLPTN(1,L) = 1.d0
      PLPTN(2,L) = Es2
      PLPTN(3,L) = 0.d0
      PLPTN(4,L) = PLPTN(4,Lp)
      PLPTN(5,L) = PLPTN(5,Lp)
      PLPTN(6,L) = PLPTN(6,Lp)
      PLPTN(7,L) = pp
      PLPTN(10,L) = 1.d0

      NLPTN(1,L) = NLPTN(1,Lp)
      NLPTN(2,L) = L-Lp
      NLPTN(3,L) = 0
      NLPTN(4,L) = 0
      NLPTN(5,L) = 0
      NLPTN(6,L) = 1

      if( ifqed.eq.2 .and. iabs( NLPTN(1,L) ).le.5 ) then
         idmain= NLPTN(1,L)
         ipmain= L
         iph1= 1
      else
         idmain= 0
         ipmain= 0
      endif

      NAD = NTOP
2     CONTINUE
      IF( NLPTN(5,NAD) .EQ. 0) THEN
         CALL QPGENf(NAD,wgt)
         Weight= Weight*wgt
         if( wgt.le.0.d0 ) return
         CALL QPXfin(NAD,wgt)
         Weight= Weight*wgt
         if( wgt.le.0.d0 ) return
C  "Backward rejection"
         if( NLPTN(3,NAD).ge.2 ) then
            qsq= PLPTN(2,NAD)  ! Q2 determined in QPGENf.
            pmoth= PLPTN(7,NAD)  ! has to be set previously.
            epmoth= sqrt( pmoth**2 +qsq ) +pmoth  ! E+p of the mother
            zmin= qsq/( epmoth**2 +qsq )
            ip1= NAD +NLPTN(4,NAD)
            ip2= ip1 +1
            if( ip2.ne.NTOP ) then
               print *,'QCDPSf: PS record is inconsistent.'
               stop
            endif
            zk= min( PLPTN(1,ip1), PLPTN(1,ip2) )
            irej= 0
            if( zk.lt.zmin ) irej= 1
            if( irej.gt.0 ) then
C  Delete the branch, but keep the Q2 value to proceed the evolution.
               idrad= NLPTN(1,NTOP)
               NLPTN(3,NAD)= 0
               NLPTN(4,NAD)= 0
               NLPTN(5,NAD)= 0
               do ip=NTOP-1,NTOP
                  do i=1,10
                     PLPTN(i,ip)= 0.d0
                     NLPTN(i,ip)= 0
                  enddo
                  if( ip.eq.ipmain ) ipmain= NAD
               enddo
               NTOP= NTOP -2
               NAD= NAD -1  ! decrement the pointer to try again.
C  Return immediately if the forced photon is rejected.
               if( ifqed.eq.2 .and. idrad.eq.22 ) then
                  Weight= 0.d0
                  return
               endif
            else
C  If accepted, set the momenta of branch products tentatively.
               ptsq= zk*( 1.d0 -zk )*qsq
               do i=1,2
                  ip= ip1 -1 +i
                  zbr= PLPTN(1,ip)
                  pl= ( zbr*epmoth -ptsq/zbr/epmoth )/2.d0
                  PLPTN(7,ip)= sqrt( pl**2 +ptsq )
               enddo
            endif
         endif
      ENDIF
      NAD = NAD + 1
      IF( NAD .LE. NTOP) GOTO 2

      RETURN
      END
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE QPGENf(NAD,weight)
C
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      common/qptest/CggF
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPinfo/Qps,ipps(3,2)
      common/QPforce/pphff(5),q2ph(4),ephmin,nphgen,ipmain,idmain,nph1
      common/QPforce1/iph1
*
      weight= 1.d0

      ID  = NLPTN(1,NAD)
      X   =    PLPTN(1,NAD)
      Q21 =abs(PLPTN(2,NAD))
      Q22 = Qps**2
      if( Q21.le.Q22 ) goto 2

*********************** 
* Start Parton Shower *
*********************** 
C QCD branch
      T1=log(Q21/QCDLd**2)
      if     (ID.eq.21) then
       Ceps=CggF*Cg*(log(1/epsfin)-11.d0/12.d0)+Tr*2*Nf/3.d0
      else if(ID.ne.21) then
C       Ceps=Cf*(2*log(1/epsfin)- 3.d0/2.d0) 
       Ceps=Cf*( 2.d0*log(1/epsfin) -3.d0/2.d0 +2.d0*epsfin
     >        -epsfin**2/2.d0 ) 
      end if
      eta=QPRAND(Idum)
      T2=T1*eta**(beta/2.d0/Ceps)
      Q2out=Exp(T2)*QCDLd**2

C Try QED if required.
      iradqed= 0
      if( ifqed.eq.1 .and. ID.ne.21 ) then
         idabs= iabs( ID )
         if( mod( idabs, 2 ).eq.1 ) then
            ch2= (1.d0/3.d0)**2
         else
            ch2= (2.d0/3.d0)**2
         endif
         ceps_e= ch2*( 2.d0*log( 1./epsqed ) -3.d0/2.d0 +2.d0*epsqed 
     >        -epsqed**2/2.d0 )
         eta_e= QPRAND(Idum)
         alpha= QPalpha( Q2l )
         t2_e= T1 +2.d0*pi/alpha/ceps_e*log( eta_e )
         if( t2_e.gt.T2 ) then
            iradqed= 1
            T2= t2_e
            Q2out=Exp(T2)*QCDLd**2
         endif
C  Photon radiation is forced when ifqed = 2.
      elseif( ifqed.eq.2 .and. NAD.eq.ipmain 
     >        .and. Q2out.le.q2ph(iph1) ) then
         iradqed= 1
         Q2out= q2ph(iph1)
         if( iph1.lt.nph1 ) then
            iph1= iph1 +1
         else
            ipmain= 0 ! clear the pointer if required photons are radiated.
         endif
      endif

C Terminate PS if the next scale is smaller than the cutoff.
      if( Q2out.lt.Q22 ) then
         PLPTN(2,NAD)= Q22
         goto 2
      endif
*
* Compornent determination
*
      Y2max=1
      Y2min=epsfin
      if     (ID.ne.21) then
*
* Quark branch part
*
       Idiag=1
       LID=ID
       NID=21
       if( iradqed.gt.0 ) NID= 22

      else if(ID.eq.21) then
*
* Gluon branch part
*
* determination of diagnal branch or not
       fqg=2*Nf*Tr/3.d0
       fgg=CggF*Cg*(log(1/epsfin)-11.d0/12.d0)
c      Write(6,*)fgg/(fgg+fqg)
       if(QPRAND(Idum).lt.fgg/(fgg+fqg)) then
        Idiag=1
       else
        Idiag=0
       end if

       if(Idiag.eq.0) then
*  Pqg compornent
        LID=MIN(QPRAND(Idum)*Nf+1,Nf*1.01d0)
        if(QPRAND(Idum).gt.0.5d0) LID=-LID
        NID=-LID
       else
*  Pgg compornent
        LID=21
        NID=21
       end if
      end if
*
* Fill the information array.
*
* Parent
*
      PLPTN(2,NAD) = Q2out
      NLPTN(3,NAD) = 2
      NLPTN(4,NAD) = (NTOP+1) - NAD
      NLPTN(5,NAD) = 1
*
* 1st product
*
      L = NTOP+1
      PLPTN(2,L) = Q2out
      NLPTN(1,L) = LID
      NLPTN(2,L) = L - NAD
      NLPTN(3,L) = 0
      NLPTN(5,L) = 0
      NLPTN(6,L) = 1
      NLPTN(8,L) = Idiag

      if( NAD.eq.ipmain ) ipmain= L
*
* 2nd product
*
      L = NTOP+2
      PLPTN(2,L) = Q2out
      NLPTN(1,L) = NID
      NLPTN(2,L) = L - NAD
      NLPTN(3,L) = 0
      NLPTN(5,L) = 0
      if( NID.eq.22 ) NLPTN(5,L) = 2    ! photon
      NLPTN(6,L) = 1
      NLPTN(8,L) = Idiag

      NTOP = NTOP + 2

      return
*
* end of "Branch" part
***********************************
* not branch
*
2     continue
      NLPTN(3,NAD) = 0
      NLPTN(4,NAD) = 0
      NLPTN(5,NAD) = 1

      return 
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE QPXfin(NAD,weight)
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPforce/pphff(5),q2ph(4),ephmin,nphgen,ipmain,idmain,nph1
*
      Fqg(x)=(1+(2-4*x+sqrt(5-16*x+16*x**2))**(-1.d0/3.d0)
     .         -(2-4*x+sqrt(5-16*x+16*x**2))**( 1.d0/3.d0))/2.d0
      Pgg0(x)=-x*(12+x*(2*x-3))/6.d0-Log(1-x)+Log(x)

      weight= 1.d0
*
* If no branch, ...
      IF( NLPTN(3,NAD).eq.0 ) then
         NLPTN(5,NAD)= 2
         return
      Endif
*
*  This parton branches.
      NPR = NAD
      L1  = NAD +NLPTN(4,NAD)
      L2  = L1 + 1
      ID1 = NLPTN(1,NPR)
      ID2 = NLPTN(1,L1)
      ID3 = NLPTN(1,L2)
*
*New X and Compornent determination
*
      Y1max=1-epsfin
      eps1= epsfin
      eta=QPRAND(Idum)
      if     (ID1.ne.21) then
*
* Quark branch part
*
         if( ID3.eq.22 ) then
            Y1max= 1-epsqed
            eps1= epsqed
C  If the photon radiation is forced, ...
            if( ifqed.eq.2 ) then
               q2br= PLPTN(2,NPR)
               pmoth= PLPTN(7,NPR)
               emoth= sqrt( pmoth**2 +q2br )
C  If minimum photon energy is given, ...
               if( ephmin.gt.0.d0 ) then
                  eps2= ephmin/pmoth -q2br/2.d0/pmoth/( emoth +pmoth )
               else
                  eps2= 0.d0
               endif
C  Forward condition.
               eps3= q2br/( ( emoth +pmoth )**2 +q2br )
               if( eps3.gt.eps2 ) eps2= eps3
C  Weighting.
               if( eps2.gt.epsqed ) then
                  if( eps2.lt.1.d0 ) then
                     ceps= 2.d0*log( 1./epsqed ) -3.d0/2.d0 
     >                    +2.d0*epsqed -epsqed**2/2.d0
                     ceps2= 2.d0*log( 1./eps2 ) -3.d0/2.d0 
     >                    +2.d0*eps2 -eps2**2/2.d0
                     weight= weight*ceps2/ceps
                     Y1max= 1 -eps2
                     eps1= eps2
                  else
                     weight= 0.d0
                     Y1max= 0.d0
                     eps1= 1.d0
                  endif
               endif
            endif
         endif

         if( weight.gt.0.d0 ) then
            fac=-2*Log(eps1)-(1-eps1)*(3-eps1)/2
            Y0=1-Exp(-eta*fac/2)
            Delta=-Log(Y0)/10000
 10         Continue
            Pqq=(-2*Log(1-Y0)-Y0*(2+Y0)/2)/fac
            If(Pqq.lt.eta) Then
               Y0=Y0+Delta
               Goto 10
            End If 
            If(Y0.gt.Y1max) Y0=Y1max
         else
            Y0= 1.d0
         endif

      else if(ID1.eq.21) then
*
* Gluon branch part
*
* determination of diagnal branch or not
        if(ID1.ne.ID2) then
*  Pqg compornent
         Y0=Fqg(eta)
        else
*  Pgg compornent
         fac=Pgg0(1-eps1)-Pgg0(eps1)
         eta=eta/2.d0+0.5d0
         Y0=1-Exp(-eta*fac/2)
         Delta=-Log(Y0)/1000
12       Continue
         Pgg=(Pgg0(Y0)-Pgg0(eps1))/fac
         If(Pgg.gt.eta) Then
          Y0=Y0-Delta
          Goto 12
         End If 
         If(Y0.gt.Y1max) Y0=Y1max
        end if
      end if
*
      YL= Y0
      YN= 1 -Y0

      PLPTN(1,L1) = YL
      PLPTN(10,L1) = 1.

      PLPTN(1,L2) = YN
      PLPTN(10,L2) = 1.

      NLPTN(5,NPR) = 2

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      double precision function sudakov_e(id,q2l,q2h)
C  QED Sudakov form factor for quarks.
C     id: PDG code of the parton.
C     q2l, q2h: lower and higher value of Q^2; q2l < q2h.
C  The splitting parameter "z" is assumed to be allowed in the range 
C  between 0 and 1 - "epsqed".
C  An exact integration is used in order to allow non-small "epsqed".
C  6 Sep 2011, S. Odaka
      implicit real*8(a-h,o-z)
      include 'qpincl.h'

      sudakov_e= 1.d0
      idabs= iabs( id )
      if( idabs.lt.1 .or. idabs.gt.6 ) return

      if( mod( idabs, 2 ).eq.1 ) then
         ch2= (1.d0/3.d0)**2
      else
         ch2= (2.d0/3.d0)**2
      endif

      ceps= 2.d0*log( 1./epsqed ) -3.d0/2.d0 +2.d0*epsqed 
     >     -epsqed**2/2.d0
      alpha= QPalpha( Q2l )
      prad= ch2*alpha/2.d0/pi*ceps*log( q2h/q2l )

      sudakov_e= exp( -prad )

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine addphoto( ip, iforce, weight )
C  Add a photon radiation to a parton according to a fragmentation 
C  function.
C  7 Sep 2011, S. Odaka
      implicit real*8(a-h,o-z)
      include 'qpincl.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPinfo/Qps,ipps(3,2)
      common/QPforce/pphff(5),q2ph(4),ephmin,nphgen,ipmain,idmain,nph1
      parameter ( rpt_ave= 1.d0 )

      weight= 1.d0

C  Determine the momentum fraction of the photon.
C  Nothing is done if no radiation.
      if( iforce.eq.0 ) then
         qsq= Qps**2
         xmin= epsqed
         id= NLPTN(1,ip)
         call xfragph( qsq, xmin, id, xphoto )
         if( xphoto.lt.xmin ) return
      elseif( iforce.eq.1 ) then
         id= NLPTN(1,ip)
         xmin= -1.d0
         if( ephmin.gt.0.d0 ) then
            xmin= ephmin/sqrt( PLPTN(4,ip)**2 
     >           +PLPTN(5,ip)**2 +PLPTN(6,ip)**2 )
         endif
         if( xmin.lt.1.d0 ) then
            call ffphgen( 1, xmin, id, xphoto, wgt )
         else
            weight= 0.d0
            return
         endif
         weight= weight*wgt
      else
         return
      endif

C  Properties of the mother.
      ptmoth= PLPTN(4,ip)**2 +PLPTN(5,ip)**2
      pmoth= sqrt( ptmoth +PLPTN(6,ip)**2 )
      ptmoth= sqrt( ptmoth )
      csphi= PLPTN(4,ip)/ptmoth
      snphi= PLPTN(5,ip)/ptmoth
      csth= PLPTN(6,ip)/pmoth
      snth= ptmoth/pmoth

C  Add a pt kick.
      rgauss= 0.d0
      do i=1,12
         rgauss= rgauss +QPRAND( idum )
      enddo
      rgauss= rgauss -6.d0
      rpt= rgauss*rpt_ave
      rphi= pi*QPRAND( idum )
      rpx= rpt*cos( rphi )
      rpy= rpt*sin( rphi )

C  Remaining parton.
      px= rpx
      py= rpy
      pz= pmoth*( 1.d0 -xphoto )
      NTOP= NTOP +1
      PLPTN(1,NTOP)= 1.d0 -xphoto
      PLPTN(2,NTOP)= 0.d0
      PLPTN(3,NTOP)= rpt**2
      PLPTN(4,NTOP)= px*csphi*csth -py*snphi +pz*csphi*snth
      PLPTN(5,NTOP)= px*snphi*csth +py*csphi +pz*snphi*snth
      PLPTN(6,NTOP)= -px*snth                +pz*csth
      PLPTN(7,NTOP)= sqrt( px**2 +py**2 +pz**2 +ptnmas( id )**2 )
      PLPTN(10,NTOP)= 1.d0
      NLPTN(1,NTOP)= id
      NLPTN(2,NTOP)= NTOP -ip
      NLPTN(3,NTOP)= 0
      NLPTN(4,NTOP)= 0
      NLPTN(5,NTOP)= 5
      NLPTN(6,NTOP)= 1
      NLPTN(7,NTOP)= 1
      NLPTN(8,NTOP)= 1

C  Photon.
      px= -rpx
      py= -rpy
      pz= pmoth*xphoto
      NTOP= NTOP +1
      PLPTN(1,NTOP)= xphoto
      PLPTN(2,NTOP)= 0.d0
      PLPTN(3,NTOP)= rpt**2
      PLPTN(4,NTOP)= px*csphi*csth -py*snphi +pz*csphi*snth
      PLPTN(5,NTOP)= px*snphi*csth +py*csphi +pz*snphi*snth
      PLPTN(6,NTOP)= -px*snth                +pz*csth
      PLPTN(7,NTOP)= sqrt( px**2 +py**2 +pz**2 )
      PLPTN(10,NTOP)= 1.d0
      NLPTN(1,NTOP)= 22
      NLPTN(2,NTOP)= NTOP -ip
      NLPTN(3,NTOP)= 0
      NLPTN(4,NTOP)= 0
      NLPTN(5,NTOP)= 5
      NLPTN(6,NTOP)= 1
      NLPTN(7,NTOP)= 1
      NLPTN(8,NTOP)= 1

C  Change to the mother
      NLPTN(3,ip)= 2
      NLPTN(4,ip)= NTOP -ip -1
      NLPTN(5,ip)= 3

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine xfragph( qsq, xmin, id, xphoto )
C  Return the momentum fraction (x) of the fragmentation photon.
C  (input) qsq: squared energy scale in GeV^2.
C          xmin: minimum "x" value.
C          id: PDG ID of the parton.
C  (output) xphoton: "x" value; =0 if no radiation.
C  7 Sep 2011, S. Odaka
      implicit real*8(a-h,o-z)

      xphoto= 0.d0
      xtry= QPRAND( idum )
      if( xtry.lt.xmin ) return

      call ffphoto( qsq, id, xtry, prob )
      ytry= QPRAND( idum )
      if( ytry.lt.prob ) xphoto= xtry

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine ffphgen( mode, xmin, idpart, xgen, weight )
C  Returns the "x" value of a photon to be radiated from a parton according to 
C  a fragmentation function, in order to supplement the radiation at Q2 scales 
C  below the PS energy scale.
C  Since a photon is always radiated, the total radiation probability has to
C  be taken into cinsideration elsewhere.
C  Note: No radiation for gluons.
C        Quarks and anti-quarks are not distinguished.
C  (input) mode =0: initialization; integrate the fragmentation probability.
C                   The other arguments are not used.
C               =1: generation for the parton "idpart".
C          idpart: PDG particle code of a parton.
C  (output) xgen: "x" value of the photon to be generated.
C  20 Oct 2011, S. Odaka
      implicit real*8(a-h,o-z)
      include 'qpincl.h'
      common/QPinfo/Qps,ipps(3,2)
      common/QPforce/pphff(5),q2ph(4),ephmin,nphgen,ipmain,idmain,nph1
      parameter ( nx= 100 )
      real*8 ffinteg(nx,5)
      save ffinteg

      weight= 1.d0

C  Initialization.
      if( mode.eq.0 ) then
         qsq= Qps**2
         x0= epsqed
         xwid= ( 1.d0 -x0 )/nx
         do id=1,5
            ffintpre= 0.d0
            ffpre= 0.d0
            do ix=1,nx
               xval= x0 +ix*xwid
               if( xval.gt.1.d0 ) xval= 1.d0
               call ffphoto( qsq, id, xval, ffval )
               ffinteg(ix,id)=  ffintpre +( ffpre +ffval )/2.d0*xwid
               ffintpre= ffinteg(ix,id)
               ffpre= ffval
            enddo
            pphff(id)= ffinteg(nx,id)
         enddo
C  Generation.
      elseif( mode.eq.1 ) then
         xgen= 0.d0
         id= iabs( idpart )
         if( id.gt.5 ) return
         x0= epsqed
         xwid= ( 1.d0 -x0 )/nx
         ffintmin= 0.d0
         ixmin= 0
C  If a hard photon is required, ...
         if( xmin.gt.x0 ) then
            rxmin= ( xmin -x0 )/xwid
            ixmin= rxmin
            dix= rxmin -ixmin
            if( ixmin.gt.0 ) then
               ffintmin= ffinteg(ixmin,id) 
     >              +( ffinteg(ixmin+1,id) -ffinteg(ixmin,id) )*dix
            else
               ffintmin= ffinteg(1,id)*dix
            endif
            weight= weight*( pphff(id) -ffintmin )/pphff(id)
         endif
C  Random generation.
         ffinteg0= ffintmin +QPRAND( idum )*( pphff(id) -ffintmin )
         do ix=ixmin+1,nx
            if( ffinteg(ix,id).gt.ffinteg0 ) goto 1
         enddo
         ix= nx
 1       continue
         if( ix.gt.1 ) then
            dinteg= ffinteg(ix,id) -ffinteg(ix-1,id)
            if( abs( dinteg )/pphff(id).gt.1.d-12 ) then
               dix= ( ffinteg(ix,id) -ffinteg0 )/dinteg
            else
               dix= 0.d0
            endif
         else
            dix= ( ffinteg(ix,id) -ffinteg0 )/ffinteg(ix,id)
         endif
         xgen= x0 +( ix -dix )*xwid
      else
         print *,'ffphgen: illegal mode =',mode
         stop
      endif

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine ffphoto( qsq, id, xval, prob )
C  Return the fragmentation probability of partons to a photon.
C  (input) qsq: squared energy scale in GeV^2.
C          id: PDG ID of the parton.
C          xval: "x" value.
C  (output) prob: fragmentation probability.
C  19 Oct 2011, S. Odaka
      implicit real*8(a-h,o-z)
      parameter ( iflag= 1 )

C  Fragmentation function to photons by Bourhis, Fontannaz, and Guillet 
C  [Eur. Phys. J. C 2, 529 (1998)].
C  The program was copied from the DIPHOX 1.2 package.
      call fonfra(xval,iflag,qsq,xdup,xdubp,xddp,xddbp,xdsp,
     >        xdcp,xdbp,xdbbp,xdtp,xdtbp,xdgp)
      if( id.eq.1 ) then
         prob= xddp/xval
      elseif( id.eq.-1 ) then
         prob= xddbp/xval
      elseif( id.eq.2 ) then
         prob= xdup/xval
      elseif( id.eq.-2 ) then
         prob= xdubp/xval
      elseif( id.eq.3 ) then
         prob= xdsp/xval
      elseif( id.eq.-3 ) then
         prob= xdsp/xval
      elseif( id.eq.4 ) then
         prob= xdcp/xval
      elseif( id.eq.-4 ) then
         prob= xdcp/xval
      elseif( id.eq.5 ) then
         prob= xdbp/xval
      elseif( id.eq.-5 ) then
         prob= xdbbp/xval
      elseif( id.eq.6 ) then
         prob= xdtp/xval
      elseif( id.eq.-6 ) then
         prob= xdtbp/xval
      elseif( id.eq.21 ) then
         prob= xdgp/xval
      else
         print *,'ffphoto: unsupported parton id = ',id
         stop
      endif
      if( prob.lt.0.d0 ) prob= 0.d0    ! for safety.

      return
      end
