*****************************************************************************
* QCDPS program library V1.5  final state PS
* Auther : 
*  V1.0: Based on the Kato-note and QEDPS made by Munehisa (Yamanashi Univ.)
*                                         Y. Kurihara (KEK)   03/Aug./2001
*  V1.3: Tanaka-Munehisa algorithm  for singlet-part
*    Mod. Phys. Lett. A13 (1998) 1085,    Y. Kurihara (KEK)   10/Dec./2001
*  V1.3 completed                         Y. Kurihara (KEK)   26/Dec./2001
*  V1.4: X-deterministic PS               Y. Kurihara (KEK)   27/Dec./2001
*  V1.4.2:x-determination improved        Y. Kurihara (KEK)   14/Jan./2002
*  V1.4.3:Fix bug in first x fixing.      Y. Kurihara (KEK)   06/Mar./2002
*  V1.4.4:Fix bug in QPXfix.              Y. Kurihara (KEK)   11/Mar./2002
*  V1.4.5:Boost up non-diagonal branch    Y. Kurihara (KEK)   20/Oct./2003
*  V1.4.6:fix bug in non-siglet treatment Y. Kurihara (KEK)   22/Oct./2003
*  V1.4.7:fix the particle replacement       S. Odaka (KEK)   24/Jul./2007
*  V1.4.8:migration to GR@PPA                S. Odaka (KEK)   25/Feb./2008
*     New commons QPinfo, QPlock, PSKfail were added. The min. energy 
*     scale "Qps" is now able to be refered externally. A new steering 
*     routine "runQCDPS" was made to implement additional functions.
*     "grcpdf1" is used to refer the starting PDF.
*  V1.4.9:QPsetLd added in QPINIT            S. Odaka (KEK)   10/Mar./2008
*     This subroutine import the Lambda value from PDF if possible.
*     Users can choose another value using the same routine later.
*  V1.4.10:further bug fix for non-diagonals S. Odaka (KEK)   18/Mar./2008
*  V1.4.11:fix bug:real variable in array argument. YK(KEK)   08/Sep./2009
*
*  V1.5.0:Final state PS is added. Y. Kurihara (KEK)          08/Sep./2009
*  V1.5.1: The final-state PS was separated to QCDPSf.f.
*                                                  S.O.(KEK)   22 Oct 2009
*  V1.5.2: A separate cutoff "epsfin" for FSR.     S.O.(KEK)    7 May 2010
*  V1.5.3: Qps was increased from 4.6 to 5.0.      S.O.(KEK)   14 Apr 2011
*  V1.5.4: The common QPmass was added.            S.O.(KEK)   15 Apr 2011
*  V1.5.5: New random number generator (randgen).  S.O.(KEK)   28 Jun 2011
*  V1.5.6: The initialization of beta was moved above the line to call 
*          QPsetLd in QPINIT.                      S.O.(KEK)   23 Aug 2011
*  V1.6.0: QED FSR is supported.                   S.O.(KEK)   20 Sep 2011
*  V1.6.1: A scheme to enforce QED FSR added       S.O.(KEK)   14 Dec 2011
*  V1.6.2: Forced multiple QED FSR supported.      S.O.(KEK)   27 Apr 2012
*****************************************************************************
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE QPINIT(Eb)
      IMPLICIT REAL*8 (A-H,O-Z)
C
      character Version*40
      parameter(version='V1.6.2 (27/Apr/2012)')
C
      include 'qpincl.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
C         PLPTN( 1, ):  The lightcone fraction of this branch
C         PLPTN( 2, ):  virtual mass squared
C         PLPTN( 3, ):  Pt**2
C         PLPTN( 4, ):  x-component of the four momentum
C         PLPTN( 5, ):  y-component of the four momentum
C         PLPTN( 6, ):  z-component of the four momentum
C         PLPTN( 7, ):  E-component of the four momentum
C         PLPTN( 8, ):  + component of the lightcone momentum
C         PLPTN( 9, ):  - component of the lightcone momentum
C         PLPTN(10, ):  Weight of this branch
C
C         NLPTN( 1, ): particle ID
C         NLPTN( 2, ): relative address of the parent
C         NLPTN( 3, ): number of the children
C         NLPTN( 4, ): relative address of the first child
C         NLPTN( 5, ): status of the process
C         NLPTN( 6, ): spacelike(-1) or timelike(+1)
C         NLPTN( 7, ): non-singlet: 1 , singlet: 0
C         NLPTN( 8, ): diagonal branch: 1, non-diagonal branch 0
      common/QPinfo/Qps,ipps(3,2)
      common/QPlock/xpdf_lock(2),iqplock
      common/PSKfail/ ifail_pskini,nfail_pskini
      common/QPmass/glmass,qkmass(6)
C
      write(6,'('' QCDPS '',A40)') Version
*
      Ebeam=Eb
C Minimum energy scale (GeV) of PS
      Qps= 5.0d0
C For PSKINI
      ifail_pskini= 0
      nfail_pskini= 0
C Make sure that QCDPS is unlocked at the start.
      iqplock= 0

      print '(5X,''Ebeam '',F10.3,'' GeV'')',Ebeam
      print '(5X,''Cutoff energy scale '',F10.3,'' GeV'')',Qps

* Random number seed
      ISEED = 123456789
      call QPRINI( ISEED )

* Initial Q2 for PDF 
      Q2PDF(0) = Qps**2
      Q2PDF(1) = Qps**2
      Q2PDF(2) = Qps**2
      Q2PDF(3) = Qps**2
      Q2PDF(4) = Qps**2
      Q2PDF(5) = Qps**2

C beta0
      beta = 11-2.d0/3.d0*Nf
      print '(5X,''beta for alpha_s '',F8.4)',beta
C Lambda from PDF
C PDF must have be initialized.
      call QPsetLd( -1.d0 )

C      aLmd2=1.d-6
      aLmd2=1.d-12

C  Cutoff for ISR
      eps= 1.d-6
C  Cutoff for FSR
      epsfin= 1.d-3
C  Cutoff for QED FSR
      epsqed= 5.d-2
      print '('' Cutoff parameters in PS'')'
      print '(5X,''ISR '',E8.2,'', FSR(QCD) '',E8.2,'', FSR(QED)'',
     >     E8.2)',eps,epsfin,epsqed
C  Flag for QED FSR (0:off, 1:on, 2:force)
      ifqed= 0

      do 1 l=1,10000
      do 1 i =1,10
         plptn(i,l) = 0.0d0
         nlptn(i,l) = 0
 1    continue
      NTOP=0

C  Parton masses to be used in PS kinematics routines.
      glmass= refmas( 0 )
      do i=1,6
         qkmass(i)= refmas( i )
      enddo
      write(6,*) 'Parton masses are set as'
      write(6,'(''   gluon '',F8.3,'', quarks '',6F8.3)') glmass,qkmass
      print*,''

      RETURN
      END

      double precision function refmas( id )
C  This function was made because the variable "PI" is defined in both 
C  "inclm.h" and "qpincl.h".
      IMPLICIT REAL*8 (A-H,O-Z)
      include 'inclm.h'
      if( id.eq.0 ) then
         refmas= amg
      elseif( id.eq.1 ) then
         refmas= amdq(1)
      elseif( id.eq.2 ) then
         refmas= amuq(1)
      elseif( id.eq.3 ) then
         refmas= amdq(2)
      elseif( id.eq.4 ) then
         refmas= amuq(2)
      elseif( id.eq.5 ) then
         refmas= amdq(3)
      elseif( id.eq.6 ) then
         refmas= amuq(3)
      endif
      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE QPEVINI
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPlock/xpdf_lock(2),iqplock

      if( iqplock.eq.1 ) return
      if( NTOP.le.0 ) return

      do 1 l=1,NTOP
      do 1 i =1,10
       plptn(i,l) = 0.0d0
       nlptn(i,l) = 0
1     continue
      NTOP=0

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine runQCDPS( iproton, iparton, idir, scale, xxx, xpdf )
C  Interafce to QCDPS.
C  (input) iproton: =1 for proton, and -1 for anti-proton
C          iparton: PDG code of the final parton
C          idir: =1 for +z direction, and -1 for -z direction
C          scale: PS energy scale (GeV)
C          xxx: x value
C  (output) xpdf = x * pdf(x,Q)
C  Feb. 22, 2008, S. Odaka
C  Sep. 08, 2009, Y. Kurihara : for v1.5
      IMPLICIT REAL*8(A-H,O-Z)

      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPinfo/Qps,ipps(3,2)
      common/QPlock/xpdf_lock(2),iqplock
      COMMON /QPIF/Iif

C  Qps: minimum energy scale (GeV) of PS given in QPINIT
C  ipps(j,i): j'th pointer for the i'th beam (i = 1,2)
C       j = 1 starting point (beam particle)
C           2 final colliding parton
C           3 end point (usually a radiation)
C  xpdf_lock(i): x*pdf value previously evaluated
C                i = 1 for idir = 1, and 2 for idir = -1
C  iqplock: flag to lock the QPLIST contents
C           = 1 to lock the contents and return the previous evaluation.
*
* Intial state PS (space-like evolution)
*
      Iif =-1
*
      if( idir.gt.0 ) then
         ibeam= 1
      else
         ibeam= 2
      endif

      if( iqplock.eq.1 ) then
         xpdf= xpdf_lock(ibeam)
         return
      endif

      npre= NTOP
      call QCDPS(  iproton, iparton, idir, scale, xxx, pdf )

C  Finally remaining parton of this PS.
C  The last one is the final parton if no radiation.
      ipfin= NTOP -1
      if( ( NTOP -npre ).lt.4 ) ipfin= NTOP
C  Check the flavor of the final parton.
      if( NLPTN(1,ipfin).ne.iparton ) pdf= 0.d0

C  Set the pointers.
      ipps(1,ibeam)= npre +1
      ipps(2,ibeam)= ipfin
      ipps(3,ibeam)= NTOP

      xpdf= xxx*pdf
      xpdf_lock(ibeam)= xpdf

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine runQCDPSb
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'grchad.inc'
C...Include HEP common parameter.
      INTEGER MAXPUP
      PARAMETER (MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON/HEPRUP/IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),XMAXUP(MAXPUP),
     &LPRUP(MAXPUP)
      SAVE /HEPRUP/

      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      call QPEVINI
      if( IDBMUP(1).gt.0 ) then
         call QCDPSb( 1, IDUP(1), 1, GRCFAQ, GPRI(5) )
      else
         call QCDPSb(-1, IDUP(1), 1, GRCFAQ, GPRI(5) )
      endif
      if( IDBMUP(2).gt.0 ) then
         call QCDPSb( 1, IDUP(2),-1, GRCFAQ, GPRI(6) )
      else
         call QCDPSb(-1, IDUP(2),-1, GRCFAQ, GPRI(6) )
      endif

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE runQCDPSf( weight )
C  Apply the final-state PS to the events generated by GR@PPA.
C  The information of a hard interaction event is copied from the LHA common.
C  The event must be the one before applying the initial-state PS.
C  Oct 27 2009, S. Odaka
C
C  Those lines for the momentum adjustment to ensure the energy conservation 
C  was removed. Thus, the energy is not conserved, while the momentum is 
C  strictly conserved.
C  Jul 28 2011, S. Odaka
C
C  Modified to support the QED FSR.
C  Sep 20 2011, S.O.
C
C  A scheme to enforce QED radiation was supported.
C  An argunmet was added to return the total event weight.
C  Dec 14 2011, S.O.
C
C  Forced multiple-photon radiation (up to 4) was supported.
C  Apr 26 2012, S.O.
C
C  Formulae in "QPfset" for the forced photon-radiation were corrected.
C  May 2 2012, S.O.
C
      IMPLICIT REAL*8(A-H,O-Z)
      include 'grchad.inc'
      include 'qpincl.h'
      common/QPforce/pphff(5),q2ph(4),ephmin,nphgen,ipmain,idmain,nph1
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPcolor/icptn(2,10000)

      parameter ( ncmax= 10 )
      integer ipcands(ncmax),ifcands(ncmax)

C  Flag for the corrections.
      logical corshat/.true./
      logical corpdf/.true./

      weight= 1.d0

C  Setup; copy the hard interaction info.
      if( ishower.eq.0 ) call QPEVINI
      if( NTOP.gt.0 ) then
         do ip=1,NTOP
            icptn(1,ip)= 0
            icptn(2,ip)= 0
         enddo
      endif
      iphard= NTOP +1
      call lha2ps
      iptop= NTOP

C  Final-state PS.
      if( ishwfin.gt.0 ) then
         call scale_fsr( Escale )

C  Turn on the QED FSR if requested.
         if( ishwfin.eq.2 ) ifqed= 1   ! mixed PS

C  Determine which parton should radiate, if the QED radiation is forced.
C  ifcands(i) = number of photons to be radiated from the i'th candidate
C  parton.
         if( ishwfin.eq.3 ) then
            if( nphgen.lt.1 .or. nphgen.gt.4 ) then
               print *,'runQCDPSf: nphgen = ',nphgen
               stop
            endif
C  Find candidate partons which may radiate photons.
            ncands= 0
            do ip=iphard,iptop
               if( NLPTN(5,ip).eq.11 ) then
                  idabs= iabs( NLPTN(1,ip) )
                  if( idabs.ge.1 .and. idabs.le.5 ) then
                     if( ncands.lt.ncmax ) then
                        ncands= ncands +1
                        ipcands(ncands)= ip
                        ifcands(ncands)= 0
                     endif
                  endif
               endif
            enddo
C  Assign the number of photons to be radiated from each candidate.
            do iph=1,nphgen
               icand= ncands*QPRAND( idum ) +1.d0
               if( icand.lt.1 ) icand= 1
               if( icand.gt.ncands ) icand= ncands
               ifcands(icand)= ifcands(icand) +1
            enddo
            weight= weight * ncombd( nphgen, ncands )
         endif

C  Time-like branches.
         wsave= weight
 3       continue
         weight= wsave
         do ip=iphard,iptop
            if( NLPTN(5,ip).eq.11 ) then
               if( ishwfin.eq.3 ) then
                  ifqed= 0
                  do icand=1,ncands
                     if( ip.eq.ipcands(icand) .and.
     >                    ifcands(icand).ge.1 ) then
                        ifqed= 2
                        nph1= ifcands(icand)
                        call QPfset( ip, Escale, wgt )
                        weight= weight*wgt
                        if( wgt.le.0.d0 ) return
                     endif
                  enddo
               endif
               call QCDPSf( ip, Escale, wgt)
               weight= weight*wgt
               if( wgt.le.0.d0 ) return
               istart= ip +NLPTN(4,ip)
               call pskfin( istart )
C  Add fragmentation photon at smaller Q^2.
               nfin= NTOP
               if( ifqed.eq.1 ) then
                  do ip1=istart,nfin
                     if( NLPTN(5,ip1).eq.5 ) then
                        idabs= iabs( NLPTN(1,ip1) )
                        if( idabs.ge.1 .and. idabs.le.5 ) then
                           call addphoto( ip1, 0, wgt )
                        endif
                     endif
                  enddo
               elseif( ifqed.eq.2 .and. ipmain.gt.0 ) then
                  do ip1=istart,nfin
                     if( ip1.eq.ipmain ) then
                        call addphoto( ip1, 1, wgt )
                        weight= weight*wgt
                        if( wgt.le.0.d0 ) return
                        ipmain= 0
                        if( NLPTN(1,ip1).ne.idmain ) then
                           call QPDUMP
                           print *,'Inconsistent id =',NLPTN(1,ip1)
                           print *,'whereas idmain =',idmain
                           stop
                        endif
                        goto 1
                     endif
                  enddo
                  call QPDUMP
                  print *,'Main parton not found; ipmain = ',ipmain
                  stop
 1                continue
               endif
            endif
         enddo

C  Correction due to the increase of the total energy.
         if( corshat .or. corpdf ) then
            wgtcor= 1.d0
            shat0= GPRI(5)*GPRI(6)*GRCECM**2
            pxsum= 0.d0
            pysum= 0.d0
            pzsum= 0.d0
            esum= 0.d0
            do ip=iphard,NTOP
               if( NLPTN(3,ip).eq.0 ) then
                  pxsum= pxsum +PLPTN(4,ip)
                  pysum= pysum +PLPTN(5,ip)
                  pzsum= pzsum +PLPTN(6,ip)
                  esum= esum +PLPTN(7,ip)
               endif
            enddo
            shat= esum**2 -pxsum**2 -pysum**2 -pzsum**2
            if( corshat ) wgtcor= wgtcor * shat0/shat
            if( corpdf ) then
               xfac= sqrt( shat/shat0 )
               wpdf0= factpdf( GPRI(5), GPRI(6) )
               x1= GPRI(5)*xfac
               x2= GPRI(6)*xfac
               if( x1.gt.1.d0 ) x1= 1.d0
               if( x2.gt.1.d0 ) x2= 1.d0
               wpdf= factpdf( x1, x2 )
               wgtcor= wgtcor * wpdf/wpdf0
            endif
            if( QPRAND( idum ).gt.wgtcor ) then
               do ip=iptop+1,NTOP
                  do i=1,10
                     NLPTN(i,ip)= 0
                     PLPTN(i,ip)= 0.d0
                  enddo
               enddo
               NTOP= iptop
               goto 3
            endif
         endif

C  Reset the color information of added partons.
         if( NTOP.gt.iptop ) then
            do ip=iptop+1,NTOP
               icptn(1,ip)= 0
               icptn(2,ip)= 0
            enddo
         endif

C  For safety,
         ifqed= 0

      endif

C  Copy supplementary info; e.g., decay products.
      call lha2ps_supp( iphard )

C  Boost to the lab frame.
      call psfbst( iphard )

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      integer function ncombd( m_in, n_in )
C  Return the number of combinations for selecting m objects out of n, 
C  in which any duplication is allowed.
C  The number is equal to the so-called "Homogeneous Product", which 
C  is expressed as H(n,m) = (n+m-1)!/[(n-1)!m!].
C  The method in this routine must be faster than the direct calculation 
C  of this equation since n and m are always small in our use.
C  20 Apr. 2012, S. Odaka
      parameter ( n_max= 5, m_max= 4 )
      integer ncomb(m_max,n_max)
      logical first/.true./
      save ncomb,first

      if( first ) then
         do n=1,n_max
            ncomb(1,n)= n
         enddo
         do m=1,m_max
            ncomb(m,1)= 1
         enddo
         do n=2,n_max
            do m=2,m_max
               ncomb(m,n)= ncomb(m,n-1) +ncomb(m-1,n)
            enddo
         enddo
         first= .false.
      endif

      if( m_in.gt.m_max .or. n_in.gt.n_max ) then
         print *,'ncombd: input is too large.'
         stop
      endif
      ncombd= ncomb(m_in,n_in)

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      double precision function factpdf( x1, x2 )
      implicit double precision (a-h,o-z)
      INCLUDE 'grchad.inc'
      include 'inclk.h'

      call grcpdf1( kfcode(1), x1, GRCFAQ, xpdf1 )
      kf2= kfcode(2)
      if( IGPP.ne.0 .and. iabs( kf2 ).le.5 ) kf2= -kf2
      call grcpdf1( kf2, x2, GRCFAQ, xpdf2 )

      factpdf= xpdf1*xpdf2/x1/x2
      if( factpdf.lt.0.d0 ) factpdf= 0.d0

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine QPfset( ip, Escale, weight )
C  Set the Q^2 at which photons should be radiated.
C  Q^2 = -1.0 for the FF (fragmentation function) radiation.
      implicit double precision (a-h,o-z)
      include 'qpincl.h'
      common/QPinfo/Qps,ipps(3,2)
      common/QPforce/pphff(5),q2ph(4),ephmin,nphgen,ipmain,idmain,nph1
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP

      weight= 1.d0

      idabs= iabs( NLPTN(1,ip) )
      if( idabs.ge.1 .and. idabs.le.5 ) then   ! for safety
         q2h= Escale**2
         q2l= Qps**2
         if( mod( idabs, 2 ).eq.1 ) then
            ch2= (1.d0/3.d0)**2
         else
            ch2= (2.d0/3.d0)**2
         endif
         ceps= ch2*( 2.d0*log( 1./epsqed ) 
     >        -3.d0/2.d0 +2.d0*epsqed -epsqed**2/2.d0 )
         do iph=1,nph1
            cqed= QPalpha( q2h )*ceps/2.d0/pi
            sudps= exp( -cqed*log( q2h/q2l ) )   ! Sudakov form factor
            pradps= 1.d0 -sudps             ! PS radiation probability
            if( iph.lt.nph1 ) then
               eta= 1.d0 -pradps*QPRAND( idum )
               q2ph(iph)= q2h*exp( log( eta )/cqed )
               weight= weight*pradps
               q2h= q2ph(iph)
            else     ! consider the FF radiation in the last radiation.
               pradff= sudps*( 1.d0 -exp( -pphff(idabs) ) )
               pradtot= pradps +pradff
               shareps= pradps/pradtot
               if( QPRAND( idum ).le.shareps ) then
                  eta= 1.d0 -pradps*QPRAND( idum )
                  q2ph(iph)= q2h*exp( log( eta )/cqed )
               else
                  q2ph(iph)= -1.d0
               endif
               weight= weight*pradtot
            endif
         enddo
      else
         weight= 0.d0
      endif

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine psfbst( iphard )
      IMPLICIT REAL*8 (A-H,O-Z)

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

C  For Parton Shower
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPinfo/Qps,ipps(3,2)
C  ipps(j,i): j'th pointer for the i'th beam (i = 1,2)
C       j = 1 starting point (beam particle)
C           2 final colliding parton
C           3 end point (usually a radiation)

      real*8 bdir(3),ptemp(4)

C  Define the Lorentz boost from the initial-state parton momenta.
C  Use the momenta in the LHA common if the initial-state PS is not activated.
      if( iphard.gt.1 ) then
         ip1= ipps(2,1)
         ip2= ipps(2,2)
         call bstcm( PLPTN(4,ip1), PLPTN(4,ip2),
     >        pcm, ecm1, ecm2, bdir, gamma, phi, theta )
         pini2= ( PLPTN(4,ip1) +PLPTN(4,ip2) )**2
     >         +( PLPTN(5,ip1) +PLPTN(5,ip2) )**2
     >         +( PLPTN(6,ip1) +PLPTN(6,ip2) )**2
      else
         call bstcm( PUP(1,1), PUP(1,2),
     >        pcm, ecm1, ecm2, bdir, gamma, phi, theta )
         pini2= ( PUP(1,1) +PUP(1,2) )**2
     >     +( PUP(2,1) +PUP(2,2) )**2 +( PUP(3,1) +PUP(3,2) )**2
      endif

C  Redefine the gamma factor.
C  Since the energy is not necessarily conserved now, we have to redefine it 
C  in order to make the momentum conserved.
      ecm= 0.
      do ip=iphard,NTOP
         nchild= NLPTN(3,ip)
         if( nchild.eq.0 ) then
            ecm= ecm +PLPTN(7,ip)
         endif
      enddo
      gamma= sqrt( pini2/ecm**2 +1.d0 )

C  Boost the final-state particles from the cm frame to the lab. frame.
      do ip=iphard,NTOP
         do i=1,4
            ptemp(i)= PLPTN(3+i,ip)
         enddo
         call bstbk( ptemp, bdir, gamma, phi, theta, PLPTN(4,ip) )         
      enddo

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      double precision function QPalpha( q2 )
      IMPLICIT REAL*8(A-H,O-Z)
      include 'inclm.h'

      QPalpha= ALPHA

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      FUNCTION alphaQCD(Q2)
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
*
      alphaQCD = 4*pi/beta/log(Q2/QCDLd**2)
*
      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      FUNCTION sudakov(ID1,q22,q21)
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      COMMON /QPIF/Iif
      common/qptest/CggF
*
      If     (Iif.eq. 1) Then
C       CggF=1.0d0
       CggF=2.0d0
       eps1= epsfin
      Else If(Iif.eq.-1) then
       CggF=2.0d0
       eps1=eps
      Else
       print *,'sudakov: invalid Iif ',Iif
       Stop
      End If
*

      T1=log(q21/QCDLd**2)
      T2=log(q22/QCDLd**2)

      T0=log(aLmd2/QCDLd**2)
*
      if     (ID1.eq.21) then
c      P2=2.d0/beta*2*Cg*( (T2-T1)-(T0+11.d0/12.d0)*log(T2/T1) )
c      P2=2.d0/beta  *Cg*( (T2-T1)-(T0+11.d0/12.d0)*log(T2/T1) )
c    .+   2.d0/beta*2*Nf*Tr/3.d0*log(T2/T1)
*Constant upper bound ***********
       P2=2.d0/beta*(log(T2/T1))*
     .(CggF*Cg*(log(1/eps1)-11.d0/12.d0)+2*Nf*Tr/3.d0)
*********************************
      else if(ID1.ne.21) then
c      P2=2*Cf/beta*(2*(T2-T1)-log(T2/T1)*(2*T0+1.5d0))
*Constant upper bound ***********
       P2=2*Cf/beta*(log(T2/T1))*(2*log(1/eps1)-3.d0/2.d0)
*********************************
      end if

c     sudakov=exp(-p2)
      sudakov=exp(Iif*p2)

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE QPDUMP

      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      DIMENSION PSUM(3)

      WRITE(6,101)NTOP
 101  FORMAT(' LIST OF OUTPUT, # OF LISTS=',I5)
      WRITE(6,102)
 102  FORMAT('     ID',6X,' X ',6X,'Q2',8X,'PT2',8X,'PX',8X,'PY',8X,
     &  'PZ',8X,'E',8X,'Wgt',7X,'NLPTN(I,*)=')


      DO 110 I =1,NTOP
       WRITE(6,103)I,NLPTN(1,I),(PLPTN(K,I),K=1,7),PLPTN(10,I)
     .,(NLPTN(K,I),K=2,8)
 110  CONTINUE

 103  FORMAT(' ',I3,I4,2X,8(G9.2,1X),2X,7I3)
 
      RETURN
      END
*----+---1---------2---------3---------4---------5---------6---------7-+
C------------------------------------------------------------------
C         RANDOM
C------------------------------------------------------------------
C         RANDOM NUMBER  ( 0- 1.0 , UNIFORM )
C------------------------------------------------------------------
C
      FUNCTION QPRAND( IDUM  )
      IMPLICIT REAL*8(A-H,O-Z)

 1    continue
      QPRAND= ZBQLU01(DUMMY)
      if( QPRAND.ge.1.d0 ) goto 1     ! for safety
 
      RETURN
      END
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE QPRINI( ISEED )
      INTEGER ISEED

      print *,'QPRINI: called with ISEED = ',ISEED
      call ZBQLINI( ISEED )

      RETURN
      END
*----+---1---------2---------3---------4---------5---------6---------7-+
C------------------------------------------------------------------
C  randgen.f
C  downloaded from http://www.ucl.ac.uk/~ucakarc/work/randgen.html.
C  Those parts irrelevant to this use are commented out.
C------------------------------------------------------------------
*******************************************************************
********	FILE: randgen.f				***********
********	AUTHORS: Richard Chandler		***********
********		 (richard@stats.ucl.ac.uk)	***********
********		 Paul Northrop 			***********
********		 (northrop@stats.ox.ac.uk)	***********
********	LAST MODIFIED: 26/8/03			***********
********	See file randgen.txt for details	***********
*******************************************************************

      BLOCK DATA ZBQLBD01
*
*       Initializes seed array etc. for random number generator.
*       The values below have themselves been generated using the
*       NAG generator.
*
      COMMON /ZBQL0001/ ZBQLIX,B,C
      DOUBLE PRECISION ZBQLIX(43),B,C
      INTEGER I
      DATA (ZBQLIX(I),I=1,43) /8.001441D7,5.5321801D8,
     +1.69570999D8,2.88589940D8,2.91581871D8,1.03842493D8,
     +7.9952507D7,3.81202335D8,3.11575334D8,4.02878631D8,
     +2.49757109D8,1.15192595D8,2.10629619D8,3.99952890D8,
     +4.12280521D8,1.33873288D8,7.1345525D7,2.23467704D8,
     +2.82934796D8,9.9756750D7,1.68564303D8,2.86817366D8,
     +1.14310713D8,3.47045253D8,9.3762426D7 ,1.09670477D8,
     +3.20029657D8,3.26369301D8,9.441177D6,3.53244738D8,
     +2.44771580D8,1.59804337D8,2.07319904D8,3.37342907D8,
     +3.75423178D8,7.0893571D7 ,4.26059785D8,3.95854390D8,
     +2.0081010D7,5.9250059D7,1.62176640D8,3.20429173D8,
     +2.63576576D8/
      DATA B / 4.294967291D9 /
      DATA C / 0.0D0 /
      END
******************************************************************
******************************************************************
******************************************************************
      SUBROUTINE ZBQLINI(SEED)
******************************************************************
*       To initialize the random number generator - either
*       repeatably or nonrepeatably. Need double precision
*       variables because integer storage can't handle the
*       numbers involved
******************************************************************
*	ARGUMENTS
*	=========
*	SEED	(integer, input). User-input number which generates
*		elements of the array ZBQLIX, which is subsequently used 
*		in the random number generation algorithm. If SEED=0,
*		the array is seeded using the system clock if the 
*		FORTRAN implementation allows it.
******************************************************************
*	PARAMETERS
*	==========
*	LFLNO	(integer). Number of lowest file handle to try when
*		opening a temporary file to copy the system clock into.
*		Default is 80 to keep out of the way of any existing
*		open files (although the program keeps searching till
*		it finds an available handle). If this causes problems,
*               (which will only happen if handles 80 through 99 are 
*               already in use), decrease the default value.
******************************************************************
C      INTEGER LFLNO
C      PARAMETER (LFLNO=80)
******************************************************************
*	VARIABLES
*	=========
*	SEED	See above
*	ZBQLIX	Seed array for the random number generator. Defined
*		in ZBQLBD01
*	B,C	Used in congruential initialisation of ZBQLIX
*	SS,MM,}	System clock secs, mins, hours and days
*	HH,DD }
*	FILNO	File handle used for temporary file
*	INIT	Indicates whether generator has already been initialised
*
C      INTEGER SEED,SS,MM,HH,DD,FILNO,I
      INTEGER SEED,I
C      INTEGER INIT
      DOUBLE PRECISION ZBQLIX(43),B,C
C      DOUBLE PRECISION TMPVAR1,DSS,DMM,DHH,DDD
      DOUBLE PRECISION TMPVAR1

      COMMON /ZBQL0001/ ZBQLIX,B,C
C      SAVE INIT

*
*	Ensure we don't call this more than once in a program
*
C      IF (INIT.GE.1) THEN
C       IF(INIT.EQ.1) THEN
C        WRITE(*,1)
C        INIT = 2
C       ENDIF
C       RETURN
C      ELSE
C       INIT = 1
C      ENDIF
*
*       If SEED = 0, cat the contents of the clock into a file
*       and transform to obtain ZQBLIX(1), then use a congr.
*       algorithm to set remaining elements. Otherwise take
*       specified value of SEED.
*
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*>>>>>>>	NB FOR SYSTEMS WHICH DO NOT SUPPORT THE  >>>>>>>
*>>>>>>>	(NON-STANDARD) 'CALL SYSTEM' COMMAND,    >>>>>>>
*>>>>>>>	THIS WILL NOT WORK, AND THE FIRST CLAUSE >>>>>>>
*>>>>>>>	OF THE FOLLOWING IF BLOCK SHOULD BE	 >>>>>>>
*>>>>>>>	COMMENTED OUT.				 >>>>>>>
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      IF (SEED.EQ.0) THEN
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*>>>>>>>	COMMENT OUT FROM HERE IF YOU DON'T HAVE  >>>>>>>
*>>>>>>>	'CALL SYSTEM' CAPABILITY ...		 >>>>>>>
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
C       CALL SYSTEM(' date +%S%M%H%j > zbql1234.tmp')
*
*       Try all file numbers for LFLNO to 999 
*
C       FILNO = LFLNO
C 10    OPEN(FILNO,FILE='zbql1234.tmp',ERR=11)
C       GOTO 12
C 11    FILNO = FILNO + 1
C       IF (FILNO.GT.999) THEN
C        WRITE(*,2)
C        RETURN
C       ENDIF
C       GOTO 10
C 12    READ(FILNO,'(3(I2),I3)') SS,MM,HH,DD
C       CLOSE(FILNO)
C       CALL SYSTEM('rm zbql1234.tmp')
C       DSS = DINT((DBLE(SS)/6.0D1) * B)
C       DMM = DINT((DBLE(MM)/6.0D1) * B)
C       DHH = DINT((DBLE(HH)/2.4D1) * B)
C       DDD = DINT((DBLE(DD)/3.65D2) * B)
C       TMPVAR1 = DMOD(DSS+DMM+DHH+DDD,B)
*<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
*<<<<<<<<	... TO HERE (END OF COMMENTING OUT FOR 	  <<<<<<<
*<<<<<<<<	USERS WITHOUT 'CALL SYSTEM' CAPABILITY	  <<<<<<<
*<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      ELSE
       TMPVAR1 = DMOD(DBLE(SEED),B)
      ENDIF
      ZBQLIX(1) = TMPVAR1
      DO 100 I = 2,43
       TMPVAR1 = ZBQLIX(I-1)*3.0269D4
       TMPVAR1 = DMOD(TMPVAR1,B)       
       ZBQLIX(I) = TMPVAR1
 100  CONTINUE

      print *,'ZBQLINI: initialized with SEED = ',SEED

C 1    FORMAT(//5X,'****WARNING**** You have called routine ZBQLINI ',
C     +'more than',/5X,'once. I''m ignoring any subsequent calls.',//)
C 2    FORMAT(//5X,'**** ERROR **** In routine ZBQLINI, I couldn''t',
C     +' find an',/5X,
C     +'available file number. To rectify the problem, decrease the ',
C     +'value of',/5X,
C     +'the parameter LFLNO at the start of this routine (in file ',
C     +'randgen.f)',/5X,
C     +'and recompile. Any number less than 100 should work.')
      END
******************************************************************
      FUNCTION ZBQLU01(DUMMY)
*
*       Returns a uniform random number between 0 & 1, using
*       a Marsaglia-Zaman type subtract-with-borrow generator.
*       Uses double precision, rather than integer, arithmetic 
*       throughout because MZ's integer constants overflow
*       32-bit integer storage (which goes from -2^31 to 2^31).
*       Ideally, we would explicitly truncate all integer 
*       quantities at each stage to ensure that the double
*       precision representations do not accumulate approximation
*       error; however, on some machines the use of DNINT to
*       accomplish this is *seriously* slow (run-time increased
*       by a factor of about 3). This double precision version 
*       has been tested against an integer implementation that
*       uses long integers (non-standard and, again, slow) -
*       the output was identical up to the 16th decimal place
*       after 10^10 calls, so we're probably OK ...
*
      DOUBLE PRECISION ZBQLU01,DUMMY,B,C,ZBQLIX(43),X,B2,BINV
      INTEGER CURPOS,ID22,ID43

      COMMON /ZBQL0001/ ZBQLIX,B,C
      SAVE /ZBQL0001/
      SAVE CURPOS,ID22,ID43
      DATA CURPOS,ID22,ID43 /1,22,43/

      B2 = B
      BINV = 1.0D0/B
 5    X = ZBQLIX(ID22) - ZBQLIX(ID43) - C
      IF (X.LT.0.0D0) THEN
       X = X + B
       C = 1.0D0
      ELSE
       C = 0.0D0
      ENDIF
      ZBQLIX(ID43) = X
*
*     Update array pointers. Do explicit check for bounds of each to
*     avoid expense of modular arithmetic. If one of them is 0 the others
*     won't be
*
      CURPOS = CURPOS - 1
      ID22 = ID22 - 1
      ID43 = ID43 - 1
      IF (CURPOS.EQ.0) THEN
       CURPOS=43
      ELSEIF (ID22.EQ.0) THEN
       ID22 = 43
      ELSEIF (ID43.EQ.0) THEN
       ID43 = 43
      ENDIF
*
*     The integer arithmetic there can yield X=0, which can cause 
*     problems in subsequent routines (e.g. ZBQLEXP). The problem
*     is simply that X is discrete whereas U is supposed to 
*     be continuous - hence if X is 0, go back and generate another
*     X and return X/B^2 (etc.), which will be uniform on (0,1/B). 
*
      IF (X.LT.BINV) THEN
       B2 = B2*B
       GOTO 5
      ENDIF

      ZBQLU01 = X/B2

      END
