C===================================================================
C Kinematics type : H2499
C                                          3 (COS3,PHI3)
C TAU  ...GPRI(1) <== XRUN(1)               \
C YST  ...GPRI(2) <== XRUN(2)      (TAU,YST) \ 
C Q1   ...GVAL(1) <== XRUN(3)       1 ------->*<------- 2 
C COS1 ...GVAL(2) <== XRUN(4)                  \
C PHI1 ...GVAL(3) <== XRUN(5)    (Q1,COS1,PHI1) *  non-resonant
C Q2   ...GVAL(4) <== XRUN(6)                  / \
C COS2 ...GVAL(5) <== XRUN(7)                4/   *  non-resonant
C PHI2 ...GVAL(6) <== XRUN(8)     (Q2,COS2,PHI2) / \
C COS3 ...GVAL(7) <== XRUN(9)                  5/   \6 (COS3,PHI3)
C PHI3 ...GVAL(8) <== XRUN(10)
C PDF  ...GPRI(10)<== XRUN(11)      Try all permutations of the numbering
C jet flavor      <== XRUN(12)      for the final-state particles.
C permutation     <== XRUN(13)
C===================================================================
      SUBROUTINE KVH2499(AJACOB)
      IMPLICIT REAL* 8(A-H,O-Z)
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL
      DOUBLE PRECISION XL,XU
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL

      integer nprm
      parameter ( nfin= 4, nprm= 24 )
      integer iprm(nfin,nprm)
      data iprm/
     >     1,2,3,4, 1,2,4,3, 1,3,2,4, 1,3,4,2, 1,4,2,3, 1,4,3,2,
     >     2,1,3,4, 2,1,4,3, 2,3,1,4, 2,3,4,1, 2,4,1,3, 2,4,3,1,
     >     3,1,2,4, 3,1,4,2, 3,2,1,4, 3,2,4,1, 3,4,1,2, 3,4,2,1, 
     >     4,1,2,3, 4,1,3,2, 4,2,1,3, 4,2,3,1, 4,3,1,2, 4,3,2,1 /
      parameter ( fangl= 5.d0 )
      parameter ( ymin= -5.d0, ymax= 5.d0 )

C...Initialization.
      AJACOB = 1.0D0
      IF (GTAU(INT(GTAU(1))+2).EQ.0.0D0) GTAU(INT(GTAU(1))+2) = 1.0D0
      IF (GQ12(INT(GQ12(1))+2).LT.1.0D-1) GQ12(INT(GQ12(1))+2)=1.0D16
      IF (GQ22(INT(GQ22(1))+2).LT.1.0D-1) GQ22(INT(GQ22(1))+2)=1.0D16

C   Permutation of final-state particles.
      jprm= nprm*xrun(13) +1
      jprm= min( jprm, nprm )
      do i=1,4
         ipx(i+2)= iprm(i,jprm) +2
      enddo

C...Set tau.
      P3MIN = GRCPTCUT(IPX(3)-2)
      P4MIN = GRCPTCUT(IPX(4)-2)
      P5MIN = GRCPTCUT(IPX(5)-2)
      P6MIN = GRCPTCUT(IPX(6)-2)
      E3MIN = SQRT(P3MIN**2+AMASS2(IPX(3)))
      E4MIN = SQRT(P4MIN**2+AMASS2(IPX(4)))
      E5MIN = SQRT(P5MIN**2+AMASS2(IPX(5)))
      E6MIN = SQRT(P6MIN**2+AMASS2(IPX(6)))
      TAUMN = (E3MIN+E4MIN+E5MIN+E6MIN)**2/GRCECM**2
      GTAU(2) = MAX(0.0D0,TAUMN,GTAU(2))                     ! Minimum tau.
      GTAU(INT(GTAU(1))+2) = MIN(1.0D0,GTAU(INT(GTAU(1))+2)) ! Maximum tau.
      RNA = XRUN(1)
      CALL GRCSTAU(RNA, TAU, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GPRI(1) = TAU
      AJACOB = AJACOB*AFACTOR

C...Set yst.
      GYST(2) = LOG(SQRT(TAU))               ! Minimum yst.
      GYST(INT(GYST(1))+2) = -LOG(SQRT(TAU)) ! Maximum yst.
      RNA = XRUN(2)
      CALL GRCSYST(RNA, YST, AFACTOR)
      GPRI(2) = YST
      AJACOB = AJACOB*AFACTOR

C...Set x1,x2,s,w
      GPRI(5) = SQRT(TAU)*EXP(YST)
      GPRI(6) = SQRT(TAU)*EXP(-YST)
      GPRI(3) = TAU*GRCECM**2
      GPRI(4) = SQRT(TAU)*GRCECM
C...Set beta,gamma
      GPRI(7) = TANH(YST)
      GPRI(8) = COSH(YST)

C   Set Q2
      op56min= max( grcrconcut(ipx(5)-2), grcrconcut(ipx(6)-2) )/fangl
      qsqmin= amass2(ipx(5)) +amass2(ipx(6))
     >     +2.d0*( e5min*e6min -p5min*p6min*cos( op56min ) )
      qsqmax= ( gpri(4) -e3min -e4min )**2
      qsqmin= max( gq22(2), qsqmin )
      qsqmax= min( gq22(int(gq22(1))+2), qsqmax )
      if( qsqmin.gt.qsqmax ) goto 999

      rna= xrun(6)
      gval(4)= qsqmin*( qsqmax/qsqmin )**rna
      ajacob= ajacob*gval(4)*log( qsqmax/qsqmin )/(2*pi)

C   Set Q1
      op45min= max( grcrconcut(ipx(4)-2), grcrconcut(ipx(5)-2) )/fangl
      op46min= max( grcrconcut(ipx(4)-2), grcrconcut(ipx(6)-2) )/fangl
      qsqmin= amass2(ipx(4)) +gval(4)
     >     +2.d0*( e4min*e5min -p4min*p5min*cos( op45min ) )
     >     +2.d0*( e4min*e6min -p4min*p6min*cos( op46min ) )
     >     +2.d0*( e5min*e6min -p5min*p6min*cos( op56min ) )
      qsqmax= ( gpri(4) -e3min )**2
      qsqmin= max( gq12(2), qsqmin )
      qsqmax= min( gq12(int(gq12(1))+2), qsqmax )
      if( qsqmin.gt.qsqmax ) goto 999

      rna= xrun(3)
      gval(1)= qsqmin*( qsqmax/qsqmin )**rna
      ajacob= ajacob*gval(1)*log( qsqmax/qsqmin )/(2*pi)
      if( gval(1).lt.gval(4) ) goto 999

C   Set COS1
      rna= xrun(4)
      y= ymin +( ymax -ymin )*rna
      gval(2)= tanh( y )
      ajacob= ajacob*( ymax -ymin )/cosh( y )**2

C   Set PHI1
      rna= xrun(5)
      gval(3)= 2*pi*rna
      ajacob= ajacob*2*pi

C   Set COS2
      rna= xrun(7)
      y= ymin +( ymax -ymin )*rna
      gval(5)= tanh( y )
      ajacob= ajacob*( ymax -ymin )/cosh( y )**2

C   Set PHI2
      rna= xrun(8)
      gval(6)= 2*pi*rna
      ajacob= ajacob*2*pi

C   Set COS3
      rna= xrun(9)
      y= ymin +( ymax -ymin )*rna
      gval(7)= tanh( y )
      ajacob= ajacob*( ymax -ymin )/cosh( y )**2

C   Set PHI3
      rna= xrun(10)
      gval(8)= 2*pi*rna
      ajacob= ajacob*2*pi

C...Set Flavor.
      IF (NDIM.GT.10.AND.GPRI(10).EQ.0.0D0) THEN
         GPRI(10) = XRUN(11)
      ENDIF

      RETURN
 999  CONTINUE
      IREJ = 1
      AJACOB = 0.0D0
      RETURN
      END

C------------------------
C Set 4-body kinematics.
C------------------------
      SUBROUTINE KSH2499(PF4,AJACOB)
      IMPLICIT REAL* 8(A-H,O-Z)
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      DOUBLE PRECISION PF4(4,10),AJACOB
      double precision ppa(4),pbost(4),ptemp(4)

C...beta function.
      DOUBLE PRECISION BETA
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      ajacob= 1.d0
      IF (NNF.NE.6) THEN
         WRITE(*,*) 'GRC::Error: wrong Nr. of partcle'
      ENDIF

C...Check invariant mass.
      if( gval(1).lt.( amass1(ipx(4)) +amass1(ipx(5)) 
     >     +amass1(ipx(6)) )**2 ) goto 999
      if( gval(4).lt.( amass1(ipx(5)) +amass1(ipx(6)) )**2 ) goto 999
      if( gval(1).lt.gval(4) ) goto 999

C...Set Energy and Momentum.
      E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.D0/GPRI(4)

C...Set particle 1.
      ip= ipx(1)
      PF4(1,ip) = 0.0D0
      PF4(2,ip) = 0.0D0
      PF4(3,ip) = SQRT((E1-AMASS1(1))*(E1+AMASS1(1)))
      PF4(4,ip) = E1
 
C...Set particle 2.
      ip= ipx(2)
      PF4(1,ip) = 0.0D0
      PF4(2,ip) = 0.0D0
      PF4(3,ip) = -SQRT((E2-AMASS1(2))*(E2+AMASS1(2)))
      PF4(4,ip) = E2

      do i=1,4
         ppa(i)= pf4(i,ip)
      enddo

C   Set particle 3
      ip= ipx(3)
      ecm= ( gpri(3) +amass2(ip) -gval(1) )/2.d0/gpri(4)
      pcm= sqrt( ecm**2 -amass2(ip)**2 )
      csth= gval(2)
      snth= sqrt( 1.d0 -csth**2 )
      csph= cos( gval(3) )
      snph= sin( gval(3) )
      pf4(1,ip)= pcm*snth*csph
      pf4(2,ip)= pcm*snth*snph
      pf4(3,ip)= pcm*csth
      pf4(4,ip)= ecm
       
      ajacob= ajacob*beta( amass2(ip)/gpri(3), gval(1)/gpri(3) )
     >     /(32.d0*pi**2)

C   Recoil system
      pbost(1)= -pf4(1,ip)
      pbost(2)= -pf4(2,ip)
      pbost(3)= -pf4(3,ip)
      pbost(4)= ( gpri(3) -amass2(ip) +gval(1) )/2.d0/gpri(4)

C   Particle 4
      ip= ipx(4)
      ecm= ( gval(1) +amass2(ip) -gval(4) )/2.d0/sqrt( gval(1) )
      pcm= sqrt( ecm**2 -amass2(ip)**2 )
      csth= gval(5)
      snth= sqrt( 1.d0 -csth**2 )
      csph= cos( gval(6) )
      snph= sin( gval(6) )
      pf4(1,ip)= pcm*snth*csph
      pf4(2,ip)= pcm*snth*snph
      pf4(3,ip)= pcm*csth
      pf4(4,ip)= ecm

      ptemp(1)= -pf4(1,ip)
      ptemp(2)= -pf4(2,ip)
      ptemp(3)= -pf4(3,ip)
      ptemp(4)= ( gval(1) -amass2(ip) +gval(4) )/2.d0/sqrt( gval(1) )
      call ktolab( ppa, pbost, ptemp, pf4(1,ip),
     >                         ptemp, pf4(1,ip) )
       
      ajacob= ajacob*beta( amass2(ip)/gval(1), gval(4)/gval(1) )
     >     /(32.d0*pi**2)

      do i=1,4
         pbost(i)= ptemp(i)
      enddo

C   Particles 5 and 6
      ip5= ipx(5)
      ip6= ipx(6)
      ecm5= ( gval(4) +amass2(ip5) -amass2(ip6) )/2.d0/sqrt( gval(4) )
      pcm= sqrt( ecm5**2 -amass2(ip5)**2 )
      ecm6= ( gval(4) -amass2(ip5) +amass2(ip6) )/2.d0/sqrt( gval(4) )
      csth= gval(7)
      snth= sqrt( 1.d0 -csth**2 )
      csph= cos( gval(8) )
      snph= sin( gval(8) )
      pf4(1,ip5)= pcm*snth*csph
      pf4(2,ip5)= pcm*snth*snph
      pf4(3,ip5)= pcm*csth
      pf4(4,ip5)= ecm5
      pf4(1,ip6)= -pf4(1,ip5)
      pf4(2,ip6)= -pf4(2,ip5)
      pf4(3,ip6)= -pf4(3,ip5)
      pf4(4,ip6)= ecm6

      call ktolab( ppa, pbost, pf4(1,ip6), pf4(1,ip5),
     >                         pf4(1,ip6), pf4(1,ip5) )

      ajacob= ajacob
     >     *beta( amass2(ip5)/gval(4), amass2(ip6)/gval(4) )
     >     /(32.d0*pi**2)

C...Check kinematics.
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) GO TO 999
      ENDDO

C...Set Jacobian.
      ajacob= ajacob*fact

      RETURN
 999  CONTINUE
      IREJ = 3
      AJACOB = 0.0D0
      RETURN
      END
