      subroutine pskfin( istart )
C  Determine the momenta of partons in the final-state PS.
C  Assumptions:
C    "istart" is the pointer to the starting point of a PS chain; 
C    i.e., the momenta of a hard interaction product is already copied  
C    to this array element.
C    Q^2 and z have been determined by QCDPSf and QPXfixf; i.e.
C    NLPTN(5,i) = 2 for all the intermediate states and radiations.
C  Principle:
C    pt^2 of each branch = z(1-z)Q^2
C    pl of evolving partons = z*( p of the parent parton )
C    The vector momentum is conserved but the energy is not.
C    The overall scale of the momenta should be adjusted later
C    in order to acomplish the energy conservation.
C  Status codes (NLPTN(5,i)) after the execution of this routine
C         3: intermediate states of the evolution
C         4: radiation from PS branches
C         5: parton at which the evolution was terminated
C  16 Oct 2009, S. Odaka
C
C  A modification was done to pick up the Q2 of a branch to be used to 
C  determine the pt from the record of the "parent", according to the change 
C  in QCDPSf.f.  7 May 2010, S.O.
C
C  A new definition of the lomgitudinal momentum.
C  28 May 2010, S.O.
C
      implicit real*8(a-h,o-z)
      include 'inclm.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP

C  The starting point may be the end.
      ip= istart
      if( NLPTN(3,ip).lt.2 ) then
         amrad= ptnmas( NLPTN(1,ip) )
         PLPTN(7,ip)= sqrt( PLPTN(4,ip)**2 +PLPTN(5,ip)**2
     >        +PLPTN(6,ip)**2 +amrad**2 )
         NLPTN(5,ip)= 5
         return
      endif

      do ip=istart,NTOP
C  Pick up a parton which has the status of 2 and has children.
         if( NLPTN(5,ip).eq.2 .and. NLPTN(3,ip).ge.2 ) then

C  Parent parton
            q2br= PLPTN(2,ip)
            px= PLPTN(4,ip)
            py= PLPTN(5,ip)
            pz= PLPTN(6,ip)
            pt2= px**2 +py**2
            pabs= sqrt( pt2 +pz**2 )
            pt= sqrt( pt2 )
C  Rotation angles
            phi= atan2( py, px )
            csph= cos( phi )
            snph= sin( phi)
            theta= atan2( pt, pz )
            csth= cos( theta )
            snth= sin( theta )

C  Children
            ip1= ip +NLPTN(4,ip)
            ip2= ip1 +1
C  Branching parameters
            zbr= PLPTN(1,ip1)
            pt2br= zbr*( 1.d0 -zbr )*q2br
            ptbr= sqrt( pt2br )
            PLPTN(3,ip1)= pt2br
            PLPTN(3,ip2)= pt2br

C  Parameters to randomize the orientation of the branching plane
            phibr= 2.d0*PI*QPRAND( IDUM )
            csbr= cos( phibr )
            snbr= sin( phibr )

C  Light-cone momentum of the mother.
            plight= ( pabs +sqrt( pabs**2 +q2br ) )/2.d0

C  Momenta in the frame where the parent parton goes along z-axis
            px1= csbr*ptbr
            py1= snbr*ptbr
            pz1= zbr*plight
            pz1= pz1 -pt2br/4.d0/pz1

            px2= -px1
            py2= -py1
            pz2= ( 1.d0 -zbr )*plight
            pz2= pz2 -pt2br/4.d0/pz2

C  Rotation
            PLPTN(4,ip1)= csph*csth*px1 -snph*py1 +csph*snth*pz1
            PLPTN(5,ip1)= snph*csth*px1 +csph*py1 +snph*snth*pz1
            PLPTN(6,ip1)=     -snth*px1                +csth*pz1

            PLPTN(4,ip2)= csph*csth*px2 -snph*py2 +csph*snth*pz2
            PLPTN(5,ip2)= snph*csth*px2 +csph*py2 +snph*snth*pz2
            PLPTN(6,ip2)=     -snth*px2                +csth*pz2

C  Set the energy and the status code to the products if they do not branch.
            if( NLPTN(3,ip1).lt.2 ) then
               amrad= ptnmas( NLPTN(1,ip1) )
               PLPTN(7,ip1)= sqrt( PLPTN(4,ip1)**2 +PLPTN(5,ip1)**2
     >              +PLPTN(6,ip1)**2 +amrad**2 )
               NLPTN(5,ip1)= 5
            endif
            if( NLPTN(3,ip2).lt.2 ) then
               amrad= ptnmas( NLPTN(1,ip2) )
               PLPTN(7,ip2)= sqrt( PLPTN(4,ip2)**2 +PLPTN(5,ip2)**2
     >              +PLPTN(6,ip2)**2 +amrad**2 )
               NLPTN(5,ip2)= 5
            endif

C  Done.
            NLPTN(5,ip)= 3

         endif
      enddo

      return
      end
